/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.core.collections;

import java.lang.reflect.Array;
import java.util.Iterator;
import java.util.NoSuchElementException;

import shohaku.core.collections.wrapper.UnmodifiableIterator;
import shohaku.core.collections.wrapper.WrappedIterator;
import shohaku.core.helpers.Eval;
import shohaku.core.lang.Boxing;

/**
 * <code>Iterator</code>インタフェースのユーティリティメソッドを提供します。
 */
public class IteratorUtils {

    /*
     * Empty
     */

    /* 空要素の反復子。 */
    private static class EmptyIterator implements Iterator {
        public void remove() {
            throw new UnsupportedOperationException();
        }

        public boolean hasNext() {
            return false;
        }

        public Object next() {
            throw new NoSuchElementException();
        }

    }

    /*
     * Array
     */

    private static abstract class AbstractArrayIterator implements Iterator {
        protected final int length;

        protected int index = -1;

        protected AbstractArrayIterator(Object a) {
            if (a == null) {
                throw new NullPointerException();
            }
            this.length = Array.getLength(a);
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }

        public boolean hasNext() {
            if ((index + 1) < this.length) {
                index++;
                return true;
            }
            return false;
        }

        public Object next() {
            if (0 > index) {
                throw new IllegalStateException();
            }
            if (index < this.length) {
                return nextObject(index);
            }
            throw new NoSuchElementException();
        }

        protected abstract Object nextObject(int inx);
    }

    /*
     * Object Array
     */

    private static class ObjectArrayIterator extends AbstractArrayIterator {
        private final Object[] a;

        private ObjectArrayIterator(Object[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return a[inx];
        }
    }

    private static class CharacterArrayIterator extends AbstractArrayIterator {
        private final char[] a;

        private CharacterArrayIterator(char[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return Boxing.box(a[inx]);
        }
    }

    private static class BooleanArrayIterator extends AbstractArrayIterator {
        private final boolean[] a;

        private BooleanArrayIterator(boolean[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return Boxing.box(a[inx]);
        }
    }

    private static class ByteArrayIterator extends AbstractArrayIterator {
        private final byte[] a;

        private ByteArrayIterator(byte[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return Boxing.box(a[inx]);
        }
    }

    private static class ShortArrayIterator extends AbstractArrayIterator {
        private final short[] a;

        private ShortArrayIterator(short[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return Boxing.box(a[inx]);
        }
    }

    private static class IntegerArrayIterator extends AbstractArrayIterator {
        private final int[] a;

        private IntegerArrayIterator(int[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return Boxing.box(a[inx]);
        }
    }

    private static class LongArrayIterator extends AbstractArrayIterator {
        private final long[] a;

        private LongArrayIterator(long[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return Boxing.box(a[inx]);
        }
    }

    private static class FloatArrayIterator extends AbstractArrayIterator {
        private final float[] a;

        private FloatArrayIterator(float[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return Boxing.box(a[inx]);
        }
    }

    private static class DoubleArrayIterator extends AbstractArrayIterator {
        private final double[] a;

        private DoubleArrayIterator(double[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return Boxing.box(a[inx]);
        }
    }

    /*
     * ToString Array
     */

    private static class ObjectToStringArrayIterator extends AbstractArrayIterator {
        private final Object[] a;

        private ObjectToStringArrayIterator(Object[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return String.valueOf(a[inx]);
        }
    }

    private static class CharacterToStringArrayIterator extends AbstractArrayIterator {
        private final char[] a;

        private CharacterToStringArrayIterator(char[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return String.valueOf(a[inx]);
        }
    }

    private static class BooleanToStringArrayIterator extends AbstractArrayIterator {
        private final boolean[] a;

        private BooleanToStringArrayIterator(boolean[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return String.valueOf(a[inx]);
        }
    }

    private static class ByteToStringArrayIterator extends AbstractArrayIterator {
        private final byte[] a;

        private ByteToStringArrayIterator(byte[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return String.valueOf(a[inx]);
        }
    }

    private static class ShortToStringArrayIterator extends AbstractArrayIterator {
        private final short[] a;

        private ShortToStringArrayIterator(short[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return String.valueOf(a[inx]);
        }
    }

    private static class IntegerToStringArrayIterator extends AbstractArrayIterator {
        private final int[] a;

        private IntegerToStringArrayIterator(int[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return String.valueOf(a[inx]);
        }
    }

    private static class LongToStringArrayIterator extends AbstractArrayIterator {
        private final long[] a;

        private LongToStringArrayIterator(long[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return String.valueOf(a[inx]);
        }
    }

    private static class FloatToStringArrayIterator extends AbstractArrayIterator {
        private final float[] a;

        private FloatToStringArrayIterator(float[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return String.valueOf(a[inx]);
        }
    }

    private static class DoubleToStringArrayIterator extends AbstractArrayIterator {
        private final double[] a;

        private DoubleToStringArrayIterator(double[] a) {
            super(a);
            this.a = a;
        }

        protected Object nextObject(int inx) {
            return String.valueOf(a[inx]);
        }
    }

    private static class CompositeIterator implements Iterator {
        final Iterator[] iters;

        private Iterator i;

        private int index = -1;

        private CompositeIterator(Iterator[] is) {
            this.iters = is;
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }

        public boolean hasNext() {
            while (i == null || !i.hasNext()) {
                index++;
                if (index < iters.length) {
                    i = iters[index];
                } else {
                    break;
                }
            }
            return i.hasNext();
        }

        public Object next() {
            return i.next();
        }
    }

    /**
     * 複数の反復子を合成した反復子を生成して返却します。
     * 
     * @return 複数の反復子を合成した反復子
     */
    public static Iterator compositeIterator(Iterator[] is) {
        return new CompositeIterator(is);
    }

    /**
     * 空要素の反復子を生成して返却します。
     * 
     * @return 空要素の反復子
     */
    public static Iterator emptyIterator() {
        return new EmptyIterator();
    }

    /*
     * Array Iterator
     */

    /**
     * 配列の要素を反復する更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復する更新不可の反復子
     * @throws IllegalArgumentException
     *             引数が配列以外の場合
     */
    public static Iterator arrayIterator(Object a) {
        if (a == null) {
            throw new NullPointerException();
        }
        if (Eval.isArray(a)) {
            Class type = a.getClass().getComponentType();
            if (type.isPrimitive()) {
                if (type == Character.TYPE) {
                    return asIterator((char[]) a);
                } else if (type == Boolean.TYPE) {
                    return asIterator((boolean[]) a);
                } else if (type == Byte.TYPE) {
                    return asIterator((byte[]) a);
                } else if (type == Short.TYPE) {
                    return asIterator((short[]) a);
                } else if (type == Integer.TYPE) {
                    return asIterator((int[]) a);
                } else if (type == Long.TYPE) {
                    return asIterator((long[]) a);
                } else if (type == Float.TYPE) {
                    return asIterator((float[]) a);
                } else if (type == Double.TYPE) {
                    return asIterator((double[]) a);
                }
            } else {
                return asIterator((Object[]) a);
            }
        }
        throw new IllegalArgumentException("is not Array:" + a);
    }

    /**
     * 配列の要素を反復する更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復する更新不可の反復子
     */
    public static Iterator asIterator(Object[] a) {
        return new ObjectArrayIterator(a);
    }

    /**
     * 配列の要素を反復する更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復する更新不可の反復子
     */
    public static Iterator asIterator(char[] a) {
        return new CharacterArrayIterator(a);
    }

    /**
     * 配列の要素を反復する更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復する更新不可の反復子
     */
    public static Iterator asIterator(boolean[] a) {
        return new BooleanArrayIterator(a);
    }

    /**
     * 配列の要素を反復する更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復する更新不可の反復子
     */
    public static Iterator asIterator(byte[] a) {
        return new ByteArrayIterator(a);
    }

    /**
     * 配列の要素を反復する更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復する更新不可の反復子
     */
    public static Iterator asIterator(short[] a) {
        return new ShortArrayIterator(a);
    }

    /**
     * 配列の要素を反復する更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復する更新不可の反復子
     */
    public static Iterator asIterator(int[] a) {
        return new IntegerArrayIterator(a);
    }

    /**
     * 配列の要素を反復する更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復する更新不可の反復子
     */
    public static Iterator asIterator(long[] a) {
        return new LongArrayIterator(a);
    }

    /**
     * 配列の要素を反復する更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復する更新不可の反復子
     */
    public static Iterator asIterator(float[] a) {
        return new FloatArrayIterator(a);
    }

    /**
     * 配列の要素を反復する更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復する更新不可の反復子
     */
    public static Iterator asIterator(double[] a) {
        return new DoubleArrayIterator(a);
    }

    /*
     * ToString Array Iterator
     */

    /**
     * 配列の要素を反復し文字列として返す更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復し文字列として返す更新不可の反復子
     * @throws IllegalArgumentException
     *             引数が配列以外の場合
     */
    public static Iterator toStringArrayIterator(Object a) {
        if (a == null) {
            throw new NullPointerException();
        }
        if (Eval.isArray(a)) {
            Class type = a.getClass().getComponentType();
            if (type.isPrimitive()) {
                if (type == Character.TYPE) {
                    return asToStringIterator((char[]) a);
                } else if (type == Boolean.TYPE) {
                    return asToStringIterator((boolean[]) a);
                } else if (type == Byte.TYPE) {
                    return asToStringIterator((byte[]) a);
                } else if (type == Short.TYPE) {
                    return asToStringIterator((short[]) a);
                } else if (type == Integer.TYPE) {
                    return asToStringIterator((int[]) a);
                } else if (type == Long.TYPE) {
                    return asToStringIterator((long[]) a);
                } else if (type == Float.TYPE) {
                    return asToStringIterator((float[]) a);
                } else if (type == Double.TYPE) {
                    return asToStringIterator((double[]) a);
                }
            } else {
                return asToStringIterator((Object[]) a);
            }
        }
        throw new IllegalArgumentException("is not Array:" + a);
    }

    /**
     * 配列の要素を反復し文字列として返す更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復し文字列として返す更新不可の反復子
     */
    public static Iterator asToStringIterator(Object[] a) {
        return new ObjectToStringArrayIterator(a);
    }

    /**
     * 配列の要素を反復し文字列として返す更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復し文字列として返す更新不可の反復子
     */
    public static Iterator asToStringIterator(char[] a) {
        return new CharacterToStringArrayIterator(a);
    }

    /**
     * 配列の要素を反復し文字列として返す更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復し文字列として返す更新不可の反復子
     */
    public static Iterator asToStringIterator(boolean[] a) {
        return new BooleanToStringArrayIterator(a);
    }

    /**
     * 配列の要素を反復し文字列として返す更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復し文字列として返す更新不可の反復子
     */
    public static Iterator asToStringIterator(byte[] a) {
        return new ByteToStringArrayIterator(a);
    }

    /**
     * 配列の要素を反復し文字列として返す更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復し文字列として返す更新不可の反復子
     */
    public static Iterator asToStringIterator(short[] a) {
        return new ShortToStringArrayIterator(a);
    }

    /**
     * 配列の要素を反復し文字列として返す更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復し文字列として返す更新不可の反復子
     */
    public static Iterator asToStringIterator(int[] a) {
        return new IntegerToStringArrayIterator(a);
    }

    /**
     * 配列の要素を反復し文字列として返す更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復し文字列として返す更新不可の反復子
     */
    public static Iterator asToStringIterator(long[] a) {
        return new LongToStringArrayIterator(a);
    }

    /**
     * 配列の要素を反復し文字列として返す更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復し文字列として返す更新不可の反復子
     */
    public static Iterator asToStringIterator(float[] a) {
        return new FloatToStringArrayIterator(a);
    }

    /**
     * 配列の要素を反復し文字列として返す更新不可の反復子を生成して返却します。
     * 
     * @param a
     *            反復する配列
     * @return 配列の要素を反復し文字列として返す更新不可の反復子
     */
    public static Iterator asToStringIterator(double[] a) {
        return new DoubleToStringArrayIterator(a);
    }

    /*
     * Wrapper
     */

    /**
     * 更新不可の反復子にラッピングして返却します。
     * 
     * @param i
     *            ラップする反復子
     * @return 更新不可の反復子
     */
    public static Iterator unmodifiableIterator(Iterator i) {
        return new UnmodifiableIterator(i);
    }

    /**
     * ラッピングした反復子を返却します。 実装型に固有の機能へのアクセスを制限します。
     * 
     * @param i
     *            ラップする反復子
     * @return 実装をラッピングした反復子
     */
    public static Iterator wrappedIterator(Iterator i) {
        return new WrappedIterator(i);
    }

}
