/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.collections;

import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Set;

import shohaku.core.helpers.Eval;
import shohaku.core.lang.Boxing;

/**
 * <code>KayValueIterator</code>インタフェースのユーティリティメソッドを提供します。
 */
public class KayValueIteratorUtils {

    /* 空要素の反復子。 */
    private static class EmptyKayValueIterator implements KayValueIterator {

        public Object next() {
            throw new NoSuchElementException();
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }

        public Object getKey() {
            throw new NoSuchElementException();
        }

        public Object getValue() {
            throw new NoSuchElementException();
        }

        public Object setValue(Object value) {
            throw new NoSuchElementException();
        }

        public int size() {
            return 0;
        }

        public boolean hasNext() {
            return false;
        }
    }

    /* 識別子と値の反復子の抽象実装を提供します。 */
    private static abstract class AbstractKayValueIterator implements KayValueIterator {

        public void remove() {
            throw new UnsupportedOperationException();
        }
    }

    /* マップを基とする識別子と値の反復子を提供します。 */
    private static class MapKayValueIterator extends AbstractKayValueIterator {
        private final Iterator entrys;

        private final int size;

        private Map.Entry _entry = null;

        private MapKayValueIterator(Map m) {
            Set entrySet = m.entrySet();
            this.entrys = entrySet.iterator();
            this.size = entrySet.size();
        }

        public boolean hasNext() {
            return entrys.hasNext();
        }

        public Object next() {
            if (hasNext()) {
                _next();
            } else {
                throw new NoSuchElementException();
            }
            return getKey();
        }

        public Object getKey() {
            return entry().getKey();
        }

        public Object getValue() {
            return entry().getValue();
        }

        public Object setValue(Object value) {
            return entry().setValue(value);
        }

        public int size() {
            return this.size;
        }

        private Map.Entry entry() {
            if (_entry == null) {
                throw new IllegalStateException();
            }
            return _entry;
        }

        private void _next() {
            _entry = (Map.Entry) entrys.next();
        }
    }

    /* グループを基とする識別子と値の反復子を提供します。 */
    private static class GroupKayValueIterator extends AbstractKayValueIterator {
        private final Iterator entrys;

        private final int size;

        private Group.Entry _entry = null;

        private GroupKayValueIterator(Group g) {
            Set entrySet = g.entrySet();
            this.entrys = entrySet.iterator();
            this.size = entrySet.size();
        }

        public boolean hasNext() {
            return entrys.hasNext();
        }

        public Object next() {
            if (hasNext()) {
                _next();
            } else {
                throw new NoSuchElementException();
            }
            return getKey();
        }

        public Object getKey() {
            return entry().getKey();
        }

        public Object getValue() {
            return entry().getValues();
        }

        public Object setValue(Object value) {
            throw new UnsupportedOperationException();
        }

        public int size() {
            return this.size;
        }

        private Group.Entry entry() {
            if (_entry == null) {
                throw new IllegalStateException();
            }
            return _entry;
        }

        private void _next() {
            _entry = (Group.Entry) entrys.next();
        }
    }

    /* リストを基とする識別子と値の反復子を提供します。 */
    private static class SequenceKayValueIterator extends AbstractKayValueIterator {
        private final Sequence src;

        private final int size;

        private int _index = -1;

        private SequenceKayValueIterator(Object[] src) {
            this(SequenceUtils.asSequence(src));
        }

        private SequenceKayValueIterator(List src) {
            this(SequenceUtils.asSequence(src));
        }

        private SequenceKayValueIterator(Sequence src) {
            this.src = src;
            this.size = src.size();
        }

        public boolean hasNext() {
            return ((_index + 1) < size);
        }

        public Object next() {
            if (hasNext()) {
                _next();
            } else {
                throw new NoSuchElementException();
            }
            return getKey();
        }

        public Object getKey() {
            return Boxing.box(index());
        }

        public Object getValue() {
            return src.get(index());
        }

        public Object setValue(Object value) {
            return src.set(index(), value);
        }

        public int size() {
            return this.size;
        }

        private int index() {
            if (_index == -1) {
                throw new IllegalStateException();
            }
            if (src.size() != size) {
                throw new ConcurrentModificationException();
            }
            return _index;
        }

        private void _next() {
            _index++;
        }
    }

    /* 二つのリストを基とする識別子と値の反復子を提供します。 */
    private static class IndexMappingSequenceKayValueIterator extends AbstractKayValueIterator {
        private final Sequence keys;

        private final Sequence values;

        private final int size;

        private int _index = -1;

        private IndexMappingSequenceKayValueIterator(Object[] keys, Object[] values) {
            this(SequenceUtils.asSequence(keys), SequenceUtils.asSequence(values));
        }

        private IndexMappingSequenceKayValueIterator(List keys, List values) {
            this(SequenceUtils.asSequence(keys), SequenceUtils.asSequence(values));
        }

        private IndexMappingSequenceKayValueIterator(Sequence keys, Sequence values) {
            if (keys == null || values == null) {
                throw new NullPointerException();
            }
            if (keys.size() == values.size()) {
                throw new IllegalArgumentException();
            }
            this.keys = keys;
            this.values = values;
            this.size = keys.size();
        }

        public boolean hasNext() {
            return ((_index + 1) < size);
        }

        public Object next() {
            if (hasNext()) {
                _next();
            } else {
                throw new NoSuchElementException();
            }
            return getKey();
        }

        public Object getKey() {
            return keys.get(index());
        }

        public Object getValue() {
            return values.get(index());
        }

        public Object setValue(Object value) {
            return values.set(index(), value);
        }

        public int size() {
            return this.size;
        }

        private int index() {
            if (_index == -1) {
                throw new IllegalStateException();
            }
            if (keys.size() != size || values.size() != size) {
                throw new ConcurrentModificationException();
            }
            return _index;
        }

        private void _next() {
            _index++;
        }
    }

    /* 配列を区画分割して認識する識別子と値の反復子を提供します。 */
    private static class SegmentSequenceKayValueIterator extends AbstractKayValueIterator {
        private final int size;

        private final int segment;

        private final int keyOffset;

        private final int valueOffset;

        private final int start;

        private final Sequence src;

        private int _index = -1;

        private SegmentSequenceKayValueIterator(Object[] src, int st, int segment, int koff, int voff) {
            this(SequenceUtils.asSequence(src), st, segment, koff, voff);
        }

        private SegmentSequenceKayValueIterator(List src, int st, int segment, int koff, int voff) {
            this(SequenceUtils.asSequence(src), st, segment, koff, voff);
        }

        private SegmentSequenceKayValueIterator(Sequence a, int st, int segment, int koff, int voff) {
            if (a == null) {
                throw new NullPointerException();
            }
            int size = a.size();
            if (!Eval.isRange(segment, 0, size) || !Eval.isRange(koff, 0, segment) || !Eval.isRange(voff, 0, segment)
                    || !Eval.isRange(st, 0, (size - 1))) {
                throwIllegalArgumentException(st, segment, koff, voff, size);
            }
            this.size = ((size - st) / segment);
            this.segment = segment;
            this.keyOffset = koff;
            this.valueOffset = voff;
            this.start = st;
            this.src = a;
        }

        private void throwIllegalArgumentException(int st, int segment, int koff, int voff, int size) {
            throw new IllegalArgumentException("size:" + size + ", segment:" + segment + ", keyOffset:" + koff
                    + ", valueOffset:" + voff + ", start:" + st);
        }

        public boolean hasNext() {
            return ((_index + 1) < size);
        }

        public Object next() {
            if (hasNext()) {
                _index++;
            } else {
                throw new NoSuchElementException();
            }
            return getKey();
        }

        public Object getKey() {
            return src.get(keyIndex());
        }

        public Object getValue() {
            return src.get(valueIndex());
        }

        public Object setValue(Object value) {
            return src.set(valueIndex(), value);
        }

        public int size() {
            return this.size;
        }

        private int keyIndex() {
            check();
            return ((_index * segment + start) + keyOffset);
        }

        private int valueIndex() {
            check();
            return ((_index * segment + start) + valueOffset);
        }

        private void check() {
            if (0 > _index) {
                throw new IllegalStateException();
            }
        }
    }

    /* 配列を区画分割して認識する識別子と値の反復子を提供します。 */
    private static class KeyValueSequenceKayValueIterator extends AbstractKayValueIterator {
        private final int size;

        private final Sequence src;

        private int _index = -1;

        private KeyValueSequenceKayValueIterator(Object[] src) {
            this(SequenceUtils.asSequence(src));
        }

        private KeyValueSequenceKayValueIterator(List src) {
            this(SequenceUtils.asSequence(src));
        }

        private KeyValueSequenceKayValueIterator(Sequence src) {
            if (src == null) {
                throw new NullPointerException("argument is null");
            }
            if (0 != src.size() % 2) {
                throw new IllegalArgumentException("odd number, size:" + src.size());
            }
            this.size = src.size() / 2;
            this.src = src;
        }

        public boolean hasNext() {
            return ((_index + 1) < size);
        }

        public Object next() {
            if (hasNext()) {
                _index++;
            } else {
                throw new NoSuchElementException();
            }
            return getKey();
        }

        public Object getKey() {
            return src.get(keyIndex());
        }

        public Object getValue() {
            return src.get(valueIndex());
        }

        public Object setValue(Object value) {
            return src.set(valueIndex(), value);
        }

        public int size() {
            return this.size;
        }

        private int keyIndex() {
            check();
            return (_index * 2);
        }

        private int valueIndex() {
            check();
            return (_index * 2 + 1);
        }

        private void check() {
            if (0 > _index) {
                throw new IllegalStateException();
            }
        }
    }

    /* 配列の要素を名前として、defaultValue を値として識別子と値の反復子を提供します。 */
    private static class DefaultValueKayValueIterator extends AbstractKayValueIterator {
        private final Sequence sequence;

        private final Object defaultValue;

        private int _index = -1;

        private DefaultValueKayValueIterator(Object[] src, Object defaultValue) {
            this(SequenceUtils.asSequence(src), defaultValue);
        }

        private DefaultValueKayValueIterator(List src, Object defaultValue) {
            this(SequenceUtils.asSequence(src), defaultValue);
        }

        private DefaultValueKayValueIterator(Sequence src, Object defaultValue) {
            this.sequence = src;
            this.defaultValue = defaultValue;
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }

        public Object next() {
            if (hasNext()) {
                _index++;
            } else {
                throw new NoSuchElementException();
            }
            return getKey();
        }

        public boolean hasNext() {
            return ((_index + 1) < sequence.size());
        }

        public Object getKey() {
            return sequence.get(_index).toString();
        }

        public Object getValue() {
            return defaultValue;
        }

        public Object setValue(Object value) {
            throw new UnsupportedOperationException();
        }

        public int size() {
            return sequence.size();
        }
    }

    /*
     * Empty
     */

    /**
     * 空要素の識別子と値の反復子を生成して返却します。
     * 
     * @return 空要素の<code>KayValueIterator</code>
     */
    public static KayValueIterator emptyKayValueIterator() {
        return new EmptyKayValueIterator();
    }

    /*
     * As
     */

    /**
     * マップを識別子と値の反復子を生成して返却します。 <br>
     * キーの重複は発生しません。
     * 
     * @param map
     *            基となるマップ
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator asKayValueIterator(Map map) {
        return new MapKayValueIterator(map);
    }

    /**
     * グループを識別子と値の反復子を生成して返却します。 <br>
     * キーの重複は発生しません。
     * 
     * @param group
     *            基となるグループ
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator asKayValueIterator(Group group) {
        return new GroupKayValueIterator(group);
    }

    /*
     * Sequence
     */

    /**
     * リストをインデックスをキーとして識別子と値の反復子を生成して返却します。 <br>
     * キーの重複は発生しません。
     * 
     * @param list
     *            基となるリスト
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator indexKayValueIterator(List list) {
        return new SequenceKayValueIterator(list);
    }

    /**
     * 配列をインデックスをキーとして識別子と値の反復子を生成して返却します。 <br>
     * キーの重複は発生しません。
     * 
     * @param a
     *            基となる配列
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator indexKayValueIterator(Object[] a) {
        return new SequenceKayValueIterator(a);
    }

    /*
     * Index Key
     */

    /**
     * 二つのリストを識別子と値として同一のインデックスで関連付けて識別子と値の反復子を生成して返却します。 <br>
     * キーの重複が発生する可能性があります。
     * 
     * @param keys
     *            キーのリスト
     * @param values
     *            値のリスト
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator indexMappingKayValueIterator(List keys, List values) {
        return new IndexMappingSequenceKayValueIterator(keys, values);
    }

    /**
     * 二つのリストを識別子と値として同一のインデックスで関連付けて識別子と値の反復子を生成して返却します。 <br>
     * キーの重複が発生する可能性があります。
     * 
     * @param keys
     *            キーのリスト
     * @param values
     *            値のリスト
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator indexMappingKayValueIterator(Object[] keys, Object[] values) {
        return new IndexMappingSequenceKayValueIterator(keys, values);
    }

    /*
     * Segment Mapping
     */

    /**
     * リストを２要素ずつの区画で分割して識別子と値の反復子を生成して返却します。 <br>
     * リスト順に<code>key, value, key, value...</code>で参照します。 <br>
     * <code>segmentKayValueIterator(list, 0, 0, 1, 2);</code>と同意です。 <br>
     * キーの重複が発生する可能性があります。
     * 
     * @param list
     *            基となるリスト
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator segmentKayValueIterator(List list) {
        return new KeyValueSequenceKayValueIterator(list);
    }

    /**
     * 配列を２要素ずつの区画で分割して識別子と値の反復子を生成して返却します。 <br>
     * 配列順に<code>key, value, key, value...</code>で参照します。 <br>
     * <code>segmentKayValueIterator(a, 0, 0, 1, 2);</code>と同意です。 <br>
     * キーの重複が発生する可能性があります。
     * 
     * @param a
     *            基となる配列
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator segmentKayValueIterator(Object[] a) {
        return new KeyValueSequenceKayValueIterator(a);
    }

    /**
     * リストを指定された区画で分割して識別子と値の反復子を生成して返却します。 <br>
     * <code>segmentKayValueIterator(list, 0, keyOffset, valueOffset, segment);</code>と同意です。 <br>
     * キーの重複が発生する可能性があります。
     * 
     * @param list
     *            基となるリスト
     * @param segment
     *            区画数
     * @param keyOffset
     *            キーの相対Index
     * @param valueOffset
     *            値の相対Index
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator segmentKayValueIterator(List list, int segment, int keyOffset, int valueOffset) {
        return segmentKayValueIterator(list, 0, segment, keyOffset, valueOffset);
    }

    /**
     * 配列を指定された区画で分割して識別子と値の反復子を生成して返却します。 <br>
     * <code>segmentKayValueIterator(a, 0, keyOffset, valueOffset, segment);</code>と同意です。 <br>
     * キーの重複が発生する可能性があります。
     * 
     * @param a
     *            基となる配列
     * @param segment
     *            区画数
     * @param keyOffset
     *            キーの相対Index
     * @param valueOffset
     *            値の相対Index
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator segmentKayValueIterator(Object[] a, int segment, int keyOffset, int valueOffset) {
        return segmentKayValueIterator(a, 0, segment, keyOffset, valueOffset);
    }

    /**
     * リストを指定された区画で分割して識別子と値の反復子を生成して返却します。 <br>
     * キーの重複が発生する可能性があります。
     * 
     * @param list
     *            基となるリスト
     * @param start
     *            走査の開始位置
     * @param segment
     *            区画数
     * @param keyOffset
     *            キーの相対Index
     * @param valueOffset
     *            値の相対Index
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator segmentKayValueIterator(List list, int start, int segment, int keyOffset,
            int valueOffset) {
        return new SegmentSequenceKayValueIterator(list, start, segment, keyOffset, valueOffset);
    }

    /**
     * 配列を指定された区画で分割して識別子と値の反復子を生成して返却します。 <br>
     * キーの重複が発生する可能性があります。
     * 
     * @param a
     *            基となる配列
     * @param start
     *            走査の開始位置
     * @param segment
     *            区画数
     * @param keyOffset
     *            キーの相対Index
     * @param valueOffset
     *            値の相対Index
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator segmentKayValueIterator(Object[] a, int start, int segment, int keyOffset,
            int valueOffset) {
        return new SegmentSequenceKayValueIterator(a, start, segment, keyOffset, valueOffset);
    }

    /*
     * Array element as key, value as defaultValue
     */

    /**
     * 配列の要素を名前として<code>defaultValue</code>を値として識別子と値の反復子を生成して返却します。 <br>
     * キーの重複が発生する可能性があります。
     * 
     * @param a
     *            基となる配列
     * @param defaultValue
     *            値
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator defaultValueKayValueIterator(Object[] a, Object defaultValue) {
        return new DefaultValueKayValueIterator(a, defaultValue);
    }

    /**
     * リストの要素を名前として<code>defaultValue</code>を値として識別子と値の反復子を生成して返却します。 <br>
     * キーの重複が発生する可能性があります。
     * 
     * @param list
     *            基となるリスト
     * @param defaultValue
     *            値
     * @return 生成された<code>KayValueIterator</code>
     */
    public static KayValueIterator defaultValueKayValueIterator(List list, Object defaultValue) {
        return new DefaultValueKayValueIterator(list, defaultValue);
    }

}
