/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.collections.params;

import java.util.Iterator;
import java.util.NoSuchElementException;

import shohaku.core.collections.KayValueIterator;
import shohaku.core.collections.KayValueIteratorUtils;

/**
 * 媒介変数のマッピングを格納するパラメータリストの抽象実装を提供します。
 */
abstract class AbstractParameters implements Cloneable {

    /**
     * ハッシュ値をキャッシュする双方向リンクリスト構造をもつパラメータエントリを提供します。
     */
    abstract static class AbstractEntry {

        /** エントリ名。 */
        final String name;

        /** このエントリのハッシュ値。 */
        final int hash;

        /** このエントリのインデックス。 */
        final int index;

        /** このエントリの値。 */
        Object value;

        /** バケットの次のエントリ。 */
        AbstractEntry next;

        /** 双方向リンクの後方エントリ。 */
        AbstractEntry before;

        /** 双方向リンクの前方エントリ。 */
        AbstractEntry after;

        /**
         * エントリ名とエントリ値を設定して初期化します。
         * 
         * @param name
         *            エントリ名
         * @param value
         *            エントリ値
         * @param hashCode
         *            エントリのハッシュ値
         * @param index
         *            エントリのインデックス
         */
        AbstractEntry(String name, Object value, int hashCode, int index) {
            this.name = name;
            this.hash = hashCode;
            this.index = index;
            this.value = value;
        }

        /**
         * エントリ名を返却します。
         * 
         * @return エントリ名
         */
        public String getName() {
            return this.name;
        }

        /**
         * エントリの値を返却します。
         * 
         * @return エントリの値
         */
        public Object getValue() {
            return this.value;
        }

        /**
         * このエントリのハッシュ値を返却します。
         * 
         * @return このエントリのハッシュ値
         * @see java.lang.Object#hashCode()
         */
        public int hashCode() {
            return hash;
        }

        /**
         * 引数が同値であるか検証します。 <br>
         * 実装の<code>Entry</code>型で名前と値が同一の場合のみ<code>true</code>を返却します。
         * 
         * @param o
         *            比較先のオブジェクト
         * @return 実装の<code>Entry</code>型で名前と値が同一の場合のみ<code>true</code>
         * @see java.lang.Object#equals(java.lang.Object)
         */
        abstract public boolean equals(Object o);

        /**
         * このエントリの文字列表現を返却します。 <br>
         * <code>name=value</code>
         * 
         * @return このエントリの文字列表現
         * @see java.lang.Object#toString()
         */
        public String toString() {
            return this.name + "=" + this.value;
        }
    }

    /** バケット容量の最大値を定義します。 */
    static final int MAXIMUM_CAPACITY = 1 << 30;

    /** エントリを保存するバケット。 */
    AbstractEntry[] table;

    /** 双方向リンクの末尾エントリ。 */
    AbstractEntry endEntry;

    /** パラメータリストのエントリ数。 */
    int size;

    /** パラメータリストのハッシュ値。 */
    int hashCode;

    /**
     * 空のパラメータリストを初期化します。
     */
    AbstractParameters() {
        this(KayValueIteratorUtils.emptyKayValueIterator());
    }

    /**
     * 引数の識別子と値の反復子と同エントリを持つパラメータリストを初期化します。
     * 
     * @param i
     *            基となる識別子と値の反復子
     * @throws IllegalArgumentException
     *             名前に重複が発見された場合
     */
    AbstractParameters(KayValueIterator i) {
        putAllForCreate(i);
    }

    /**
     * 初期化時に引数の識別子と値の反復子と同エントリを登録します。 <br>
     * この処理はコンストラクタから呼ばれます。 <br>
     * バケットとそのサイズが初期化されている必要があります。
     * 
     * @param kvi
     *            登録する識別子と値の反復子
     */
    void putAllForCreate(KayValueIterator kvi) {
        this.table = new AbstractEntry[capacity(kvi.size())];
        AbstractEntry _entry = null;
        AbstractEntry _beginEntry = null;
        int i = 0;
        while (kvi.hasNext()) {
            kvi.next();

            String name = (String) kvi.getKey();
            Object value = kvi.getValue();
            if (name == null) {
                throw new NullPointerException("name is null");
            }

            int hash = hash(name);
            int index = indexFor(hash, table.length);
            AbstractEntry newEntry = newEntry(name, value, hash, i);
            putEntry(index, newEntry);
            if (_entry != null) {
                _entry.after = newEntry;
                newEntry.before = _entry;
            } else {
                _beginEntry = newEntry;
            }
            _entry = newEntry;
            i++;
        }
        if (null != _entry) {
            this.endEntry = _entry;
            this.endEntry.after = _beginEntry;
            _beginEntry.before = this.endEntry;
        } else {
            this.endEntry = null;
        }
        this.size = i;
        this.hashCode = _hashCode(this);
    }

    /* インデックス位置にエントリを追加します。throws IllegalArgumentException 名前に重複が発見された場合 */
    private void putEntry(int index, AbstractEntry newEntry) {
        AbstractEntry e = table[index];
        if (e == null) {
            table[index] = newEntry;
        } else {
            while (e != null) {
                if (e.name.equals(newEntry.name)) {
                    throw new IllegalArgumentException("init Parameters overlap name. " + e.name);
                }
                if (null == e.next) {
                    e.next = newEntry;
                    break;
                }
                e = e.next;
            }
        }
    }

    /**
     * エントリを生成して返却します。 拡張クラスの拡張ポイントです。
     * 
     * @param name
     *            名
     * @param value
     *            値
     * @param hashCode
     *            ハッシュ値
     * @param index
     *            インデックス
     * @return エントリ
     */
    abstract AbstractEntry newEntry(String name, Object value, int hashCode, int index);

    /**
     * パラメータリストのエントリ数を返却します。
     * 
     * @return パラメータリストのエントリ数
     */
    public int size() {
        return this.size;
    }

    /**
     * パラメータリストの削除不可能な反復子を生成して返却します。 <br>
     * 反復子の要素は<code>AbstractParameters.AbstractEntry</code>型と為ります。
     * 
     * @return パラメータリストの削除不可能な反復子
     */
    public Iterator iterator() {
        return new ParametersIterator();
    }

    /* パラメータリストの反復子。 */
    private class ParametersIterator implements Iterator {

        /* 走査中のインデックス。 */
        private int _index = -1;

        /* 走査中のエントリ。 */
        private AbstractEntry e = endEntry;

        /* 反復する次の要素が有る場合<code>true</code>を返却します。 */
        public boolean hasNext() {
            return (e != null && (_index + 1) < size);
        }

        /* 反復する次の要素(AbstractParameters.AbstractEntry)を返却します。 */
        public Object next() {
            if (hasNext()) {
                _index++;
                e = e.after;
            } else {
                throw new NoSuchElementException();
            }
            return e;
        }

        /* <code> UnsupportedOperationException </code>を発生させます。 */
        public void remove() {
            throw new UnsupportedOperationException();
        }
    }

    /**
     * パラメータリストの値の変更不可能かつ削除不可能な反復子を生成して返却します。
     * 
     * @return パラメータリストの値の変更不可能かつ削除不可能な反復子
     */
    public KayValueIterator kayValueIterator() {
        return new ParametersKayValueIterator();
    }

    /**
     * パラメータリストの反復子。
     */
    class ParametersKayValueIterator implements KayValueIterator {

        /* 走査中のエントリ。 */
        AbstractEntry e = endEntry;

        /* 走査中のインデックス。 */
        int _index = -1;

        /* <code> UnsupportedOperationException </code>を発生させます。 */
        public void remove() {
            throw new UnsupportedOperationException();
        }

        /* 反復する次の要素が有る場合<code>true</code>を返却します。 */
        public boolean hasNext() {
            return (e != null && (_index + 1) < size);
        }

        /* 次の要素に移動しそのキーを返却します。 */
        public Object next() {
            if (hasNext()) {
                _index++;
                e = e.after;
            } else {
                throw new NoSuchElementException();
            }
            return getKey();
        }

        /* 現在の要素のキーを返却します。 */
        public Object getKey() {
            check();
            return e.name;
        }

        /* 現在の要素の値を返却します。 */
        public Object getValue() {
            check();
            return e.value;
        }

        /* 現在の要素の値を返却します。 */
        public Object setValue(Object value) {
            throw new UnsupportedOperationException();
        }

        /* 要素数を返却します。 */
        public int size() {
            return size;
        }

        void check() {
            if (_index == -1) {
                throw new IllegalStateException();
            }
            if (e == null) {
                throw new NoSuchElementException();
            }
        }
    }

    /**
     * パラメータが空の場合<code>true</code>を返却します。
     * 
     * @return パラメータが空の場合<code>true</code>
     */
    public boolean isEmpty() {
        return (this.size == 0);
    }

    /**
     * パラメータが一つのみの要素を持つ場合<code>true</code>を返却します。
     * 
     * @return パラメータが一つのみの要素を持つ場合<code>true</code>
     */
    public boolean isSingleton() {
        return (this.size == 1);
    }

    /**
     * インデックスのパラメータ名を返却します。 <br>
     * 指定された名前が存在しない場合は<code>null</code>が返されます。
     * 
     * @param index
     *            インデックス
     * @return インデックスのパラメータ名
     */
    public String getName(int index) {
        AbstractEntry e = getEntry(index);
        if (null != e) {
            return e.name;
        }
        return null;
    }

    /**
     * パラメータ名のインデックスを返却します。 <br>
     * 指定された名前が存在しない場合は<code>-1</code>が返されます。
     * 
     * @param name
     *            パラメータ名
     * @return パラメータ名のインデックス
     */
    public int getIndex(String name) {
        AbstractEntry e = getEntry(name);
        if (null != e) {
            return e.index;
        }
        return -1;
    }

    /**
     * パラメータ名のパラメータ値を返却します、名前が存在しない場合は<code>null</code>が返されます。
     * 
     * @param name
     *            パラメータ名
     * @return パラメータ値
     */
    public Object getValue(String name) {
        AbstractEntry e = getEntry(name);
        if (null != e) {
            return e.value;
        }
        return null;
    }

    /**
     * パラメータ名のパラメータ値を返却します、名前が存在しない場合は<code>defaultValue</code>が返されます。
     * 
     * @param name
     *            パラメータ名
     * @param defaultValue
     *            パラメータが存在しない場合に返却される値
     * @return 指定された名前のパラメータ値又は<code>defaultValue</code>
     */
    public Object getValue(String name, String defaultValue) {
        AbstractEntry e = getEntry(name);
        if (null != e) {
            return e.value;
        }
        return defaultValue;
    }

    /**
     * インデックスのパラメータ値を返却します、インデックスが存在しない場合は<code>null</code>が返されます。
     * 
     * @param index
     *            インデックス
     * @return 指定されたインデックスのパラメータ値
     */
    public Object getValue(int index) {
        AbstractEntry e = getEntry(index);
        if (null != e) {
            return e.value;
        }
        return null;
    }

    /**
     * インデックスのパラメータ値を返却します、インデックスが存在しない場合は<code>defaultValue</code>が返されます。
     * 
     * @param index
     *            インデックス
     * @param defaultValue
     *            パラメータが存在しない場合に返却される値
     * @return 指定されたインデックスのパラメータ値又は<code>defaultValue</code>
     */
    public Object getValue(int index, String defaultValue) {
        AbstractEntry e = getEntry(index);
        if (null != e) {
            return e.value;
        }
        return defaultValue;
    }

    /**
     * 引数の名前のパラメータエントリが存在するか検証します。 <br>
     * パラメータエントリが存在する場合のみ<code>true</code>を返却します。
     * 
     * @param name
     *            検証する名前
     * @return パラメータエントリが存在する場合のみ<code>true</code>
     */
    public boolean containsName(String name) {
        return (null != getEntry(name));
    }

    /**
     * このオブジェクトのハッシュ値を返却します。 <br>
     * 全てのエントリのハッシュ値の合計です。
     * 
     * @return このオブジェクトのハッシュ値
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return this.hashCode;
    }

    /**
     * 引数が同値であるか検証します。 <br>
     * <code>AbstractParameters</code>型でエントリが全て同一の場合のみ<code>true</code>を返却します。
     * 
     * @param o
     *            比較先のオブジェクト
     * @return <code>AbstractParameters</code>型でエントリが全て同一の場合のみ<code>true</code>
     * @see java.lang.Object#equals(java.lang.Object)
     */
    abstract public boolean equals(Object o);

    /**
     * このオブジェクトの文字列表現を返却します。 <br>
     * <code>{name=value, name=value, ...}</code>
     * 
     * @return このオブジェクトの文字列表現
     * @see java.lang.Object#toString()
     */
    abstract public String toString();

    /**
     * クローンを生成して返却します。
     * 
     * @return このオブジェクトのクローン
     * @see java.lang.Object#clone()
     */
    abstract public Object clone();

    /*
     * package
     */

    /**
     * パラメータ名のエントリを返却します（パラメータ名が存在しない場合<code>null</code>）。
     * 
     * @param name
     *            パラメータ名
     * @return パラメータ名のエントリ
     */
    AbstractEntry getEntry(String name) {
        if (!isEmpty()) {
            int h = hash(name);
            int i = indexFor(h, table.length);
            AbstractEntry e = table[i];
            while (e != null && !(e.hash == h && eq(name, e.name))) {
                e = e.next;
            }
            return e;
        } else {
            return null;
        }
    }

    /**
     * インデックスのエントリを返却します（インデックスが範囲外の場合<code>null</code>）。
     * 
     * @param index
     *            インデックス
     * @return インデックスのエントリ
     */
    AbstractEntry getEntry(int index) {
        if (isEmpty() || 0 > index || index >= size) {
            return null;
        }
        if (isSingleton() && 0 == index) {
            return this.endEntry;
        }
        if (index < (size >> 1)) {
            AbstractEntry e = endEntry.after;// start entry
            for (int i = 0; i < index; i++) {
                e = e.after;
            }
            return e;
        } else {
            AbstractEntry e = endEntry;
            for (int i = (size - 1); i > index; i--) {
                e = e.before;
            }
            return e;
        }
    }

    /*
     * static
     */

    /*
     * ハッシュ値とバケット容量に関しては今後に最適化を行う。
     */

    /* */
    static boolean eq(String x, String y) {
        return (x == y || x.equals(y));
    }

    /**
     * バケットの容量を返却します。
     * 
     * @param size
     *            格納するデータサイズ
     * @return バケットの容量
     */
    static int capacity(int size) {
        if (size <= 1) {
            return size;
        }

        int capacity = 1;
        int initialCapacity = (size - 1);
        if (initialCapacity > MAXIMUM_CAPACITY) {
            initialCapacity = MAXIMUM_CAPACITY;
        }
        while (capacity < initialCapacity) {
            capacity <<= 1;
        }
        return capacity;
    }

    /**
     * ハッシュ値とバケット容量からインデックスを算出して返却します。
     * 
     * @param h
     *            ハッシュ値
     * @param length
     *            バケット容量
     * @return インデックス
     */
    static int indexFor(int h, int length) {
        return h & (length - 1);
    }

    /**
     * エントリ名からエントリのハッシュ値を算出して返却します。
     * 
     * @param name
     *            エントリ名
     * @return エントリのハッシュ値
     */
    static int hash(String name) {
        int h = name.hashCode();
        h += ~(h << 9);
        h ^= (h >>> 14);
        h += (h << 4);
        h ^= (h >>> 10);
        return h;
    }

    /**
     * パラメータリストのハッシュ値を算出して返却します。
     * 
     * @param params
     *            パラメータリスト
     * @return ハッシュ値
     */
    static int _hashCode(AbstractParameters params) {
        int hashCode = 0;
        if (!params.isEmpty()) {
            AbstractEntry e = params.endEntry.after;
            for (int i = 0; i < params.size; i++) {
                hashCode += e.hash;
                e = e.after;
            }
        }
        return hashCode;
    }

}