/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.collections.params;

import java.io.Serializable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import shohaku.core.collections.KayValueIterator;
import shohaku.core.collections.KayValueIteratorUtils;

/**
 * 媒介変数として名前と値のマッピングを格納し、要素を固定とするデータ構造を提供します。 <br>
 * このデータの要素は名前と値の対のデータ構造であり、また各要素は０から始まる順序を持ちます。 <br>
 * そのため要素へのアクセスは名前とインデックスをキーとする事ができます。 <br>
 * コンストラクタによる初期化以降には要素は変更不可となります。 <br>
 * このデータは名前と値の対のデータの双方向リンクリスト構造を持ちます。
 */
public final class Parameters extends AbstractParameters implements Serializable {

    /** <code>serialVersionUID</code>。 */
    private static final long serialVersionUID = -2819772961569801313L;

    /**
     * <code>Parameters</code>のパラメータエントリを提供します。
     */
    public static final class Entry extends AbstractEntry {

        /**
         * エントリ名とエントリ値を設定して初期化します。
         * 
         * @param name
         *            エントリ名
         * @param value
         *            エントリ値
         * @param hashCode
         *            エントリのハッシュ値
         * @param index
         *            エントリのインデックス
         */
        Entry(String name, Object value, int hashCode, int index) {
            super(name, value, hashCode, index);
        }

        /**
         * 引数が同値であるか検証します。 <br>
         * <code>Parameters.Entry</code>型で名前と値が同一の場合のみ<code>true</code>を返却します。
         * 
         * @param o
         *            比較先のオブジェクト
         * @return <code>Parameters.Entry</code>型で名前と値が同一の場合のみ<code>true</code>
         * @see java.lang.Object#equals(java.lang.Object)
         */
        public boolean equals(Object o) {
            if (this == o) {
                return true;
            }
            if (!(o instanceof Entry)) {
                return false;
            }
            Entry e = (Entry) o;
            return (name.equals(e.getName())) && ((value != null) ? value.equals(e.getValue()) : e.getValue() == null);
        }
    }

    /**
     * 空のパラメータリストを初期化します。
     */
    public Parameters() {
        super();
    }

    /**
     * 指定された配列を名前と値を交互に持つデータとしてパラメータリストを初期化します。
     * 
     * @param nameValue
     *            名前と値を交互に持つ配列
     */
    public Parameters(Object[] nameValue) {
        super(KayValueIteratorUtils.segmentKayValueIterator(nameValue));
    }

    /**
     * 指定されたリストを名前と値を交互に持つデータとしてパラメータリストを初期化します。
     * 
     * @param nameValue
     *            名前と値を交互に持つリスト
     */
    public Parameters(List nameValue) {
        super(KayValueIteratorUtils.segmentKayValueIterator(nameValue));
    }

    /**
     * 引数のマップのキーを名前として、そのキーの値を持つパラメータリストを初期化します。
     * 
     * @param prams
     *            基となるマップ
     */
    public Parameters(Map prams) {
        super(KayValueIteratorUtils.asKayValueIterator(prams));
    }

    /**
     * 引数のパラメータリストと同エントリを持つパラメータリストを初期化します。
     * 
     * @param prams
     *            基となるパラメータリスト
     */
    public Parameters(Parameters prams) {
        super(prams.kayValueIterator());
    }

    /**
     * 引数の識別子と値の反復子と同エントリを持つパラメータリストを初期化します。
     * 
     * @param i
     *            基となる識別子と値の反復子
     */
    Parameters(KayValueIterator i) {
        super(i);
    }

    /**
     * このクラスのエントリを生成して返却します。
     * 
     * @param name
     *            名
     * @param value
     *            値
     * @param hashCode
     *            ハッシュ値
     * @param index
     *            インデックス
     * @return エントリ
     * @see shohaku.core.collections.params.AbstractParameters#newEntry(java.lang.String, java.lang.Object, int, int)
     */
    AbstractEntry newEntry(String name, Object value, int hashCode, int index) {
        return new Entry(name, value, hashCode, index);
    }

    /**
     * 引数が同値であるか検証します。 <br>
     * <code>Parameters</code>型でエントリが全て同一の場合のみ<code>true</code>を返却します。
     * 
     * @param o
     *            比較先のオブジェクト
     * @return <code>Parameters</code>型でエントリが全て同一の場合のみ<code>true</code>
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Parameters)) {
            return false;
        }
        Parameters p = (Parameters) o;
        if (size != p.size()) {
            return false;
        }
        Iterator i = iterator();
        Iterator j = p.iterator();
        boolean hasNext = (i.hasNext() && j.hasNext());
        while (hasNext) {
            Entry e = (Entry) (i.next());
            Entry e2 = (Entry) (j.next());
            if (!e.equals(e2)) {
                return false;
            }
            hasNext = (i.hasNext() && j.hasNext());
        }
        return true;
    }

    /**
     * このオブジェクトの文字列表現を返却します。 <br>
     * <code>{name=value, name=value, ...}</code>
     * 
     * @return このオブジェクトの文字列表現
     * @see java.lang.Object#toString()
     */
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("{");

        Iterator i = iterator();
        boolean hasNext = i.hasNext();
        while (hasNext) {
            Entry e = (Entry) (i.next());
            buf.append(e.name);
            buf.append('=');
            Object value = e.value;
            buf.append((value == this ? "(this Parameters)" : value));
            hasNext = i.hasNext();
            if (hasNext) {
                buf.append(", ");
            }
        }

        buf.append("}");
        return buf.toString();
    }

    /**
     * クローンを生成して返却します。
     * 
     * @return このオブジェクトのクローン
     * @see java.lang.Object#clone()
     */
    public Object clone() {
        return new Parameters(this);
    }
}