/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.core.collections.wrapper;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import java.util.ListIterator;

/**
 * 指定されたセットの変更不可能なビューを提供します。 <br>
 * <br>
 * このクラスは<code>java.util.Collections#unmodifiableList(List)</code>で返されるクラスとほぼ同等の仕様です。 <br>
 * ラッピングされたリストが直列化可能の場合は直列化可能です。
 */
public class UnmodifiableList extends UnmodifiableCollection implements List, Serializable {

    /** <code>serialVersionUID</code>。 */
    private static final long serialVersionUID = -6310084907153562776L;

    /** 基となるセット。 */
    protected final List list;

    /**
     * ラップするセットを指定して初期化します。
     * 
     * @param l
     *            ラップするセット
     */
    public UnmodifiableList(List l) {
        super(l);
        this.list = l;
    }

    /**
     * 基となるリストの同メソッドを呼び出します。
     * 
     * @see java.util.List#get(int)
     */
    public Object get(int index) {
        return list.get(index);
    }

    /**
     * 基となるリストの同メソッドを呼び出します。
     * 
     * @see java.util.List#indexOf(java.lang.Object)
     */
    public int indexOf(Object o) {
        return list.indexOf(o);
    }

    /**
     * 基となるリストの同メソッドを呼び出します。
     * 
     * @see java.util.List#lastIndexOf(java.lang.Object)
     */
    public int lastIndexOf(Object o) {
        return list.lastIndexOf(o);
    }

    /*
     * UnsupportedOperation
     */

    /**
     * <code>UnsupportedOperationException</code>を発生させます。
     * 
     * @see java.util.List#remove(int)
     */
    public Object remove(int index) {
        throw new UnsupportedOperationException();
    }

    /**
     * <code>UnsupportedOperationException</code>を発生させます。
     * 
     * @see java.util.List#add(int, java.lang.Object)
     */
    public void add(int index, Object element) {
        throw new UnsupportedOperationException();
    }

    /**
     * <code>UnsupportedOperationException</code>を発生させます。
     * 
     * @see java.util.List#addAll(int, java.util.Collection)
     */
    public boolean addAll(int index, Collection c) {
        throw new UnsupportedOperationException();
    }

    /**
     * <code>UnsupportedOperationException</code>を発生させます。
     * 
     * @see java.util.List#set(int, java.lang.Object)
     */
    public Object set(int index, Object element) {
        throw new UnsupportedOperationException();
    }

    /*
     * Unmodifiable
     */

    /**
     * 基となるリストの同メソッドの戻り値を<code>UnmodifiableList</code>でラッピングしたリストを返却します。
     * 
     * @see java.util.List#subList(int, int)
     */
    public List subList(int fromIndex, int toIndex) {
        return new UnmodifiableList(list.subList(fromIndex, toIndex));
    }

    /**
     * 基となるリストの同メソッドの戻り値を<code>UnmodifiableListIterator</code>でラッピングしたリストの反復子を返却します。
     * 
     * @see java.util.List#listIterator()
     */
    public ListIterator listIterator() {
        return new UnmodifiableListIterator(list.listIterator());
    }

    /**
     * 基となるリストの同メソッドの戻り値を<code>UnmodifiableListIterator</code>でラッピングしたリストの反復子を返却します。
     * 
     * @see java.util.List#listIterator(int)
     */
    public ListIterator listIterator(final int index) {
        return new UnmodifiableListIterator(list.listIterator(index));
    }

}
