/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.helpers;

/**
 * オブジェクトからデータを切り出す機能のヘルパーメソッド群を提供します。
 */
public class Cutter {

    /**
     * 最初の文字を切り出します。
     * 
     * @param cs
     *            処理対象の文字シーケンス
     * @return 最初の文字
     */
    public static char begin(CharSequence cs) {
        return cs.charAt(0);
    }

    /**
     * 最後の文字を切り出します。
     * 
     * @param cs
     *            処理対象の文字シーケンス
     * @return 最後の文字
     */
    public static char end(CharSequence cs) {
        return cs.charAt(cs.length() - 1);
    }

    /**
     * 文字シーケンスの両端から空白文字を削除した文字シーケンスを返却します。
     * 
     * @param cs
     *            処理対象の文字シーケンス
     * @return 両端から空白文字を削除した文字シーケンス
     */
    public static CharSequence trim(CharSequence cs) {
        int len = cs.length();
        int st = 0;
        while ((st < len) && (cs.charAt(st) <= ' ')) {
            st++;
        }
        while ((st < len) && (cs.charAt(len - 1) <= ' ')) {
            len--;
        }
        return ((st > 0) || (len < cs.length())) ? cs.subSequence(st, len) : cs;
    }

    /**
     * 文字シーケンスの先頭から空白文字を削除した文字シーケンスを返却します。
     * 
     * @param cs
     *            処理対象の文字シーケンス
     * @return 先頭から空白文字を削除した文字シーケンス
     */
    public static CharSequence trimBegin(CharSequence cs) {
        int len = cs.length();
        int st = 0;
        while ((st < len) && (cs.charAt(st) <= ' ')) {
            st++;
        }
        return (st > 0) ? cs.subSequence(st, len) : cs;
    }

    /**
     * 文字シーケンスの後方から空白文字を削除した文字シーケンスを返却します。
     * 
     * @param cs
     *            処理対象の文字シーケンス
     * @return 後方から空白文字を削除した文字シーケンス
     */
    public static CharSequence trimEnd(CharSequence cs) {
        int len = cs.length();
        while ((0 < len) && (cs.charAt(len - 1) <= ' ')) {
            len--;
        }
        return (len < cs.length()) ? cs.subSequence(0, len) : cs;
    }

    /**
     * 文字列の両端から空白文字を削除した文字列を返却します。
     * 
     * @param s
     *            処理対象の文字列
     * @return 両端から空白文字を削除した文字列
     */
    public static String trim(String s) {
        return (String) trim((CharSequence) s);
    }

    /**
     * 文字列の先頭から空白文字を削除した文字列を返却します。
     * 
     * @param s
     *            処理対象の文字列
     * @return 先頭から空白文字を削除した文字列
     */
    public static String trimBegin(String s) {
        return (String) trimBegin((CharSequence) s);
    }

    /**
     * 文字列の後方から空白文字を削除した文字列を返却します。
     * 
     * @param s
     *            処理対象の文字列
     * @return 後方から空白文字を削除した文字列
     */
    public static String trimEnd(String s) {
        return (String) trimEnd((CharSequence) s);
    }

    /**
     * 文字シーケンスの両端から指定の文字を削除した文字シーケンスを返却します。
     * 
     * @param cs
     *            処理対象の文字シーケンス
     * @param chars
     *            削除する文字の配列
     * @return 両端から指定の文字を削除した文字シーケンス
     */
    public static CharSequence trim(CharSequence cs, char[] chars) {
        int len = cs.length();
        int st = 0;
        while ((st < len) && (Eval.isContains(chars, cs.charAt(st)))) {
            st++;
        }
        while ((st < len) && (Eval.isContains(chars, cs.charAt(len - 1)))) {
            len--;
        }
        return ((st > 0) || (len < cs.length())) ? cs.subSequence(st, len) : cs;
    }

    /**
     * 文字シーケンスの先頭から指定の文字を削除した文字シーケンスを返却します。
     * 
     * @param cs
     *            処理対象の文字シーケンス
     * @param chars
     *            削除する文字の配列
     * @return 先頭から指定の文字を削除した文字シーケンス
     */
    public static CharSequence trimBegin(CharSequence cs, char[] chars) {
        int len = cs.length();
        int st = 0;
        while ((st < len) && (Eval.isContains(chars, cs.charAt(st)))) {
            st++;
        }
        return (st > 0) ? cs.subSequence(st, len) : cs;
    }

    /**
     * 文字シーケンスの後方から指定の文字を削除した文字シーケンスを返却します。
     * 
     * @param cs
     *            処理対象の文字シーケンス
     * @param chars
     *            削除する文字の配列
     * @return 後方から指定の文字を削除した文字シーケンス
     */
    public static CharSequence trimEnd(CharSequence cs, char[] chars) {
        int len = cs.length();
        while ((0 < len) && (Eval.isContains(chars, cs.charAt(len - 1)))) {
            len--;
        }
        return (len < cs.length()) ? cs.subSequence(0, len) : cs;
    }

    /**
     * 文字列の両端から指定の文字を削除した文字列を返却します。
     * 
     * @param s
     *            処理対象の文字列
     * @param chars
     *            削除する文字の配列
     * @return 両端から指定の文字を削除した文字列
     */
    public static CharSequence trim(String s, char[] chars) {
        return (String) trim((CharSequence) s, chars);
    }

    /**
     * 文字列の先頭から指定の文字を削除した文字列を返却します。
     * 
     * @param s
     *            処理対象の文字列
     * @param chars
     *            削除する文字の配列
     * @return 先頭から指定の文字を削除した文字列
     */
    public static CharSequence trimBegin(String s, char[] chars) {
        return (String) trimBegin((CharSequence) s, chars);
    }

    /**
     * 文字列の後方から指定の文字を削除した文字列を返却します。
     * 
     * @param s
     *            処理対象の文字列
     * @param chars
     *            削除する文字の配列
     * @return 後方から指定の文字を削除した文字列
     */
    public static CharSequence trimEnd(String s, char[] chars) {
        return (String) trimEnd((CharSequence) s, chars);
    }

    /**
     * 文字列の両端から指定の文字数を削除した文字列を返却します。
     * 
     * @param s
     *            処理対象の文字列
     * @param beginSize
     *            前方から切り出す文字数
     * @param endSize
     *            後方から切り出す文字数
     * @return 両端から指定の文字数を削除した文字列
     */
    public static String cut(String s, int beginSize, int endSize) {
        return s.substring(beginSize, (s.length() - endSize));
    }

    /**
     * 文字列の前方から指定の文字数と、指定の文字が発見された位置を後方位置として切り出した文字列を返却します。
     * 
     * @param s
     *            処理対象の文字列
     * @param beginSize
     *            前方から切り出す文字数
     * @param endChar
     *            後方から切り出す位置を検索する文字
     * @return 両端から指定の文字数と文字の発見位置を切り出した文字列
     */
    public static String cutIndexOf(String s, int beginSize, char endChar) {
        int endSize = Seek.indexOf(s, endChar, beginSize);
        endSize = (-1 < endSize) ? endSize : s.length();
        return s.substring(beginSize, endSize);
    }

    /**
     * 文字列の指定の文字が発見された位置を前方位置として、後方から指定の文字数を切り出した文字列を返却します。
     * 
     * @param s
     *            処理対象の文字列
     * @param beginChar
     *            前方から切り出す位置を検索する文字
     * @param endSize
     *            後方から切り出す文字数
     * @return 両端から指定の文字の発見位置と文字数を切り出した文字列
     */
    public static String cutLastIndexOf(String s, char beginChar, int endSize) {
        int beginSize = Seek.lastIndexOf(s, beginChar, 0, (s.length() - 1 - endSize));
        beginSize = (-1 < beginSize) ? beginSize + 1 : 0;
        return s.substring(beginSize, (s.length() - endSize));
    }

}
