/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.helpers;

import java.lang.reflect.Array;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import shohaku.core.collections.Group;
import shohaku.core.collections.params.Parameters;

/**
 * 値を評価する式のヘルパーメソッド群を提供します。
 */
public class Eval {

    /*
     * Blank
     */

    /**
     * 引数が<code>null</code>または空の文字シーケンスであるかを評価します。
     * 
     * @param cs
     *            評価する文字シーケンス
     * @return 引数が<code>null</code>または空の文字シーケンスの場合のみ<code>true</code>
     */
    public static boolean isBlank(CharSequence cs) {
        return (cs == null || cs.length() == 0);
    }

    /**
     * 引数が全て<code>null</code>または空の文字シーケンスであるかを評価します。
     * 
     * @param cs1
     *            評価する一番目の文字シーケンス
     * @param cs2
     *            評価する二番目の文字シーケンス
     * @return 引数が全て<code>null</code>または空の文字シーケンスの場合のみ<code>true</code>
     */
    public static boolean isAndBlank(CharSequence cs1, CharSequence cs2) {
        return (isBlank(cs1) && isBlank(cs2));
    }

    /**
     * 全ての配列の要素が<code>null</code>または空の文字シーケンスであるかを評価します。
     * 
     * @param css
     *            評価する文字シーケンスの配列
     * @return 全ての配列の要素が<code>null</code>または空の文字シーケンスの場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isAndBlank(CharSequence[] css) {
        for (int i = 0; i < css.length; i++) {
            if (!isBlank(css[i])) {
                return false;
            }
        }
        return true;
    }

    /**
     * 引数に<code>null</code>または空の文字シーケンスが含まれるか評価します。
     * 
     * @param cs1
     *            評価する一番目の文字シーケンス
     * @param cs2
     *            評価する二番目の文字シーケンス
     * @return 引数に<code>null</code>または空の文字シーケンスが含まれる場合のみ<code>true</code>
     */
    public static boolean isOrBlank(CharSequence cs1, CharSequence cs2) {
        return (isBlank(cs1) || isBlank(cs2));
    }

    /**
     * 全ての配列の要素が<code>null</code>または空の文字シーケンスが含まれるか評価します。
     * 
     * @param css
     *            評価する文字シーケンスの配列
     * @return 全ての配列の要素が<code>null</code>または空の文字シーケンスが含まれる場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isOrBlank(CharSequence[] css) {
        for (int i = 0; i < css.length; i++) {
            if (isBlank(css[i])) {
                return true;
            }
        }
        return false;
    }

    /**
     * 引数が<code>null</code>または空かスペースのみの文字シーケンスであるかを評価します。
     * 
     * @param cs
     *            評価する文字シーケンス
     * @return 引数が<code>null</code>または空かスペースのみの文字シーケンスの場合のみ<code>true</code>
     */
    public static boolean isBlankOrSpace(CharSequence cs) {
        return (cs == null || Cutter.trim(cs).length() == 0);
    }

    /**
     * 引数が全て<code>null</code>または空かスペースのみの文字シーケンスであるかを評価します。
     * 
     * @param cs1
     *            評価する一番目の文字シーケンス
     * @param cs2
     *            評価する二番目の文字シーケンス
     * @return 引数が全て<code>null</code>または空かスペースのみの文字シーケンスの場合のみ<code>true</code>
     */
    public static boolean isAndBlankOrSpace(CharSequence cs1, CharSequence cs2) {
        return (isBlankOrSpace(cs1) && isBlankOrSpace(cs2));
    }

    /**
     * 全ての配列の要素が<code>null</code>または空かスペースのみの文字シーケンスであるかを評価します。
     * 
     * @param css
     *            評価する文字シーケンスの配列
     * @return 全ての配列の要素が<code>null</code>または空かスペースのみの文字シーケンスの場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isAndBlankOrSpace(CharSequence[] css) {
        for (int i = 0; i < css.length; i++) {
            if (!isBlankOrSpace(css[i])) {
                return false;
            }
        }
        return true;
    }

    /**
     * 引数に<code>null</code>または空かスペースのみの文字シーケンスが含まれるか評価します。
     * 
     * @param cs1
     *            評価する一番目の文字シーケンス
     * @param cs2
     *            評価する二番目の文字シーケンス
     * @return 引数に<code>null</code>または空かスペースのみの文字シーケンスが含まれる場合のみ<code>true</code>
     */
    public static boolean isOrBlankOrSpace(CharSequence cs1, CharSequence cs2) {
        return (isBlankOrSpace(cs1) || isBlankOrSpace(cs2));
    }

    /**
     * 全ての配列の要素が<code>null</code>または空かスペースのみの文字シーケンスが含まれるか評価します。
     * 
     * @param css
     *            評価する文字シーケンスの配列
     * @return 全ての配列の要素が<code>null</code>または空かスペースのみの文字シーケンスが含まれる場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isOrBlankOrSpace(CharSequence[] css) {
        for (int i = 0; i < css.length; i++) {
            if (isBlankOrSpace(css[i])) {
                return true;
            }
        }
        return false;
    }

    /**
     * 引数が<code>null</code>または空の状態であるかを評価します。
     * 
     * @param os
     *            評価する配列
     * @return 引数が<code>null</code>または空の状態の場合のみ<code>true</code>
     */
    public static boolean isEmpty(Object[] os) {
        return (os == null || os.length == 0);
    }

    /**
     * 引数が<code>null</code>または空の状態であるかを評価します。
     * 
     * @param m
     *            評価するマップ
     * @return 引数が<code>null</code>または空の状態の場合のみ<code>true</code>
     */
    public static boolean isEmpty(Map m) {
        return (m == null || m.isEmpty());
    }

    /**
     * 引数が<code>null</code>または空の状態であるかを評価します。
     * 
     * @param c
     *            評価するコレクション
     * @return 引数が<code>null</code>または空の状態の場合のみ<code>true</code>
     */
    public static boolean isEmpty(Collection c) {
        return (c == null || c.isEmpty());
    }

    /**
     * 引数が<code>null</code>または空の状態であるかを評価します。
     * 
     * @param g
     *            評価するグループ
     * @return 引数が<code>null</code>または空の状態の場合のみ<code>true</code>
     */
    public static boolean isEmpty(Group g) {
        return (g == null || g.isEmpty());
    }

    /**
     * 引数が<code>null</code>または空の状態であるかを評価します。
     * 
     * @param p
     *            評価するパラメータリスト
     * @return 引数が<code>null</code>または空の状態の場合のみ<code>true</code>
     */
    public static boolean isEmpty(Parameters p) {
        return (p == null || p.isEmpty());
    }

    /*
     * Null
     */

    /**
     * 引数が全て<code>null</code>値であるかを評価します。
     * 
     * @param o1
     *            評価する一番目の値
     * @param o2
     *            評価する二番目の値
     * @return 引数が全て<code>null</code>値の場合のみ<code>true</code>
     */
    public static boolean isAndNull(Object o1, Object o2) {
        return (o1 == null && o2 == null);
    }

    /**
     * 全ての配列の要素が<code>null</code>値であるかを評価します。
     * 
     * @param os
     *            評価する配列
     * @return 全ての配列の要素が<code>null</code>値の場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isAndNull(Object[] os) {
        for (int i = 0; i < os.length; i++) {
            if (os[i] != null) {
                return false;
            }
        }
        return true;
    }

    /**
     * 引数に<code>null</code>値が含まれるか評価します。
     * 
     * @param o1
     *            評価する一番目の値
     * @param o2
     *            評価する二番目の値
     * @return 引数に<code>null</code>値が含まれる場合のみ<code>true</code>
     */
    public static boolean isOrNull(Object o1, Object o2) {
        return (o1 == null || o2 == null);
    }

    /**
     * 引数に<code>null</code>値が含まれるか評価します。
     * 
     * @param o1
     *            評価する一番目の値
     * @param o2
     *            評価する二番目の値
     * @param o3
     *            評価する三番目の値
     * @return 引数に<code>null</code>値が含まれる場合のみ<code>true</code>
     */
    public static boolean isOrNull(Object o1, Object o2, Object o3) {
        return (o1 == null || o2 == null || o3 == null);
    }

    /**
     * 引数に<code>null</code>値が含まれるか評価します。
     * 
     * @param o1
     *            評価する一番目の値
     * @param o2
     *            評価する二番目の値
     * @param o3
     *            評価する三番目の値
     * @param o4
     *            評価する四番目の値
     * @return 引数に<code>null</code>値が含まれる場合のみ<code>true</code>
     */
    public static boolean isOrNull(Object o1, Object o2, Object o3, Object o4) {
        return (o1 == null || o2 == null || o3 == null || o4 == null);
    }

    /**
     * 配列の要素に<code>null</code>値が含まれるか評価します。
     * 
     * @param os
     *            評価する配列
     * @return 配列の要素に<code>null</code>値が含まれる場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isOrNull(Object[] os) {
        for (int i = 0; i < os.length; i++) {
            if (os[i] == null) {
                return true;
            }
        }
        return false;
    }

    /*
     * Equals
     */

    /**
     * 指定の値が同値であるかを評価します。 <br>
     * 同値か双方<code>null</code>の場合<code>true</code>と評価します。
     * 
     * @param o
     *            評価基の値
     * @param o2
     *            評価先の値
     * @return 同値か双方<code>null</code>の場合<code>true</code>
     */
    public static boolean isEquals(Object o, Object o2) {
        if (o == o2) {
            return true;
        }
        return (o != null && o.equals(o2));
    }

    /**
     * 指定の値の内いずれかが同値であるかを評価します。 <br>
     * 同値か双方<code>null</code>の場合<code>true</code>と評価します。
     * 
     * @param o
     *            評価基の値
     * @param o2
     *            評価先の値
     * @param o3
     *            評価先の値
     * @return 指定の値の内いずれかが同値である場合<code>true</code>
     */
    public static boolean isOrEquals(Object o, Object o2, Object o3) {
        return (isEquals(o, o2) || isEquals(o, o3));
    }

    /**
     * 指定の値の内いずれかが同値であるかを評価します。 <br>
     * 同値か双方<code>null</code>の場合<code>true</code>と評価します。
     * 
     * @param o
     *            評価基の値
     * @param o2
     *            評価先の値
     * @param o3
     *            評価先の値
     * @param o4
     *            評価先の値
     * @return 指定の値の内いずれかが同値である場合<code>true</code>
     */
    public static boolean isOrEquals(Object o, Object o2, Object o3, Object o4) {
        return (isEquals(o, o2) || isEquals(o, o3) || isEquals(o, o4));
    }

    /**
     * 配列の全要素が一致するか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param a2
     *            評価先の配列
     * @return 配列の全要素が一致する場合<code>true</code>
     */
    public static boolean isEquals(char[] a, char[] a2) {
        if (a == a2) {
            return true;
        }
        if (a == null || a2 == null) {
            return false;
        }
        if (a.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a.length; i++) {
            if (a[i] != a2[i]) {
                return false;
            }
        }
        return true;
    }

    /**
     * 配列の全要素が一致するか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param a2
     *            評価先の配列
     * @return 配列の全要素が一致する場合<code>true</code>
     */
    public static boolean isEquals(boolean[] a, boolean[] a2) {
        if (a == a2) {
            return true;
        }
        if (a == null || a2 == null) {
            return false;
        }
        if (a.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a.length; i++) {
            if (a[i] != a2[i]) {
                return false;
            }
        }
        return true;
    }

    /**
     * 配列の全要素が一致するか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param a2
     *            評価先の配列
     * @return 配列の全要素が一致する場合<code>true</code>
     */
    public static boolean isEquals(byte[] a, byte[] a2) {
        if (a == a2) {
            return true;
        }
        if (a == null || a2 == null) {
            return false;
        }
        if (a.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a.length; i++) {
            if (a[i] != a2[i]) {
                return false;
            }
        }
        return true;
    }

    /**
     * 配列の全要素が一致するか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param a2
     *            評価先の配列
     * @return 配列の全要素が一致する場合<code>true</code>
     */
    public static boolean isEquals(short[] a, short[] a2) {
        if (a == a2) {
            return true;
        }
        if (a == null || a2 == null) {
            return false;
        }
        if (a.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a.length; i++) {
            if (a[i] != a2[i]) {
                return false;
            }
        }
        return true;
    }

    /**
     * 配列の全要素が一致するか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param a2
     *            評価先の配列
     * @return 配列の全要素が一致する場合<code>true</code>
     */
    public static boolean isEquals(int[] a, int[] a2) {
        if (a == a2) {
            return true;
        }
        if (a == null || a2 == null) {
            return false;
        }
        if (a.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a.length; i++) {
            if (a[i] != a2[i]) {
                return false;
            }
        }
        return true;
    }

    /**
     * 配列の全要素が一致するか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param a2
     *            評価先の配列
     * @return 配列の全要素が一致する場合<code>true</code>
     */
    public static boolean isEquals(long[] a, long[] a2) {
        if (a == a2) {
            return true;
        }
        if (a == null || a2 == null) {
            return false;
        }
        if (a.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a.length; i++) {
            if (a[i] != a2[i]) {
                return false;
            }
        }
        return true;
    }

    /**
     * 配列の全要素が一致するか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param a2
     *            評価先の配列
     * @return 配列の全要素が一致する場合<code>true</code>
     */
    public static boolean isEquals(float[] a, float[] a2) {
        if (a == a2) {
            return true;
        }
        if (a == null || a2 == null) {
            return false;
        }
        if (a.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a.length; i++) {
            if (a[i] != a2[i]) {
                return false;
            }
        }
        return true;
    }

    /**
     * 配列の全要素が一致するか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param a2
     *            評価先の配列
     * @return 配列の全要素が一致する場合<code>true</code>
     */
    public static boolean isEquals(double[] a, double[] a2) {
        if (a == a2) {
            return true;
        }
        if (a == null || a2 == null) {
            return false;
        }
        if (a.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a.length; i++) {
            if (a[i] != a2[i]) {
                return false;
            }
        }
        return true;
    }

    /**
     * 配列の全要素が一致するか評価します。 <br>
     * 要素は同値か双方<code>null</code>の場合<code>true</code>と評価します。
     * 
     * @param a
     *            評価基の配列
     * @param a2
     *            評価先の配列
     * @return 配列の全要素が一致する場合<code>true</code>
     */
    public static boolean isEquals(Object[] a, Object[] a2) {
        if (a == a2) {
            return true;
        }
        if (a == null || a2 == null) {
            return false;
        }
        if (a.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a.length; i++) {
            if (!isEquals(a[i], a2[i])) {
                return false;
            }
        }
        return true;
    }

    /*
     * Suffix, Prefix
     */

    /**
     * 指定の文字シーケンスの位置から前方一致するか評価します。
     * 
     * @param cs
     *            評価基の文字シーケンス
     * @param prefix
     *            評価先の接頭辞
     * @param offset
     *            評価を開始する相対インデックス
     * @return 指定の文字シーケンスの位置から前方一致する場合のみ<code>true</code>
     */
    public static boolean isStartsWith(CharSequence cs, CharSequence prefix, int offset) {
        if (isBlank(cs)) {
            return false;
        }
        return startsWith(cs, prefix, offset);
    }

    /**
     * 指定の文字シーケンスが指定の接尾辞を持つか評価します。
     * 
     * @param cs
     *            評価基の文字シーケンス
     * @param suffix
     *            評価先の接尾辞
     * @return 指定の文字シーケンスが指定の接尾辞を持つ場合のみ<code>true</code>
     */
    public static boolean isSuffix(CharSequence cs, CharSequence suffix) {
        if (isBlank(cs)) {
            return false;
        }
        return suffix(cs, suffix);
    }

    /**
     * 指定の文字シーケンスに接尾辞の内いずれかが含まれるか評価します。
     * 
     * @param cs
     *            評価基の文字シーケンス
     * @param suffixs
     *            評価先の接尾辞の配列
     * @return 指定の文字列に接尾辞の内いずれかが含まれる場合のみ<code>true</code>
     */
    public static boolean isOrSuffix(CharSequence cs, char[] suffixs) {
        if (isBlank(cs)) {
            return false;
        }
        char suffix = cs.charAt(cs.length() - 1);
        for (int i = 0; i < suffixs.length; i++) {
            if (suffix == suffixs[i]) {
                return true;
            }
        }
        return false;
    }

    /**
     * 指定の文字シーケンスに接尾辞の内いずれかが含まれるか評価します。
     * 
     * @param cs
     *            評価基の文字シーケンス
     * @param suffixs
     *            評価先の接尾辞の配列
     * @return 指定の文字シーケンスに接尾辞の内いずれかが含まれる場合のみ<code>true</code>
     */
    public static boolean isOrSuffix(CharSequence cs, CharSequence[] suffixs) {
        if (isBlank(cs)) {
            return false;
        }
        for (int i = 0; i < suffixs.length; i++) {
            if (suffix(cs, suffixs[i])) {
                return true;
            }
        }
        return false;
    }

    /**
     * 指定の文字シーケンスが指定の接頭辞を持つか評価します。
     * 
     * @param cs
     *            評価基の文字シーケンス
     * @param prefix
     *            評価先の接頭辞の配列
     * @return 指定の文字シーケンスが指定の接頭辞を持つ場合のみ<code>true</code>
     */
    public static boolean isPrefix(CharSequence cs, CharSequence prefix) {
        if (isBlank(cs)) {
            return false;
        }
        return prefix(cs, prefix);
    }

    /**
     * 指定の文字シーケンスに接頭辞の内いずれかが含まれるか評価します。
     * 
     * @param cs
     *            評価基の文字シーケンス
     * @param prefixs
     *            評価先の接頭辞の配列
     * @return 指定の文字列に接頭辞の内いずれかが含まれる場合のみ<code>true</code>
     */
    public static boolean isOrPrefix(CharSequence cs, char[] prefixs) {
        if (isBlank(cs)) {
            return false;
        }
        char prefix = cs.charAt(0);
        for (int i = 0; i < prefixs.length; i++) {
            if (prefix == prefixs[i]) {
                return true;
            }
        }
        return false;
    }

    /**
     * 指定の文字シーケンスに接頭辞の内いずれかが含まれるか評価します。
     * 
     * @param cs
     *            評価基の文字シーケンス
     * @param prefixs
     *            評価先の接頭辞の配列
     * @return 指定の文字シーケンスに接頭辞の内いずれかが含まれる場合のみ<code>true</code>
     */
    public static boolean isOrPrefix(CharSequence cs, CharSequence[] prefixs) {
        if (isBlank(cs)) {
            return false;
        }
        for (int i = 0; i < prefixs.length; i++) {
            if (prefix(cs, prefixs[i])) {
                return true;
            }
        }
        return false;
    }

    private static boolean suffix(CharSequence cs, CharSequence suffix) {
        int end = (cs.length() - suffix.length());
        return startsWith(cs, suffix, end);
    }

    private static boolean prefix(CharSequence cs, CharSequence prefix) {
        return startsWith(cs, prefix, 0);
    }

    private static boolean startsWith(CharSequence cs, CharSequence prefix, int offset) {

        int plen = prefix.length();
        if ((offset < 0) || (offset > cs.length() - plen)) {
            return false;
        }
        int to = offset;
        int po = 0;
        while (--plen >= 0) {
            if (cs.charAt(to++) != prefix.charAt(po++)) {
                return false;
            }
        }
        return true;
    }

    /*
     * Range
     */

    /**
     * 指定された数値が指定の範囲内か評価します。
     * 
     * @param n
     *            評価する数値
     * @param from
     *            範囲の最小値
     * @param to
     *            範囲の最大値
     * @return 指定の範囲内の場合のみ<code>true</code>
     */
    public static boolean isRange(int n, int from, int to) {
        if (from > to) {
            throw new IllegalArgumentException("from > to (" + from + " > " + to + ")");
        }
        return (from <= n && n <= to);
    }

    /**
     * 指定された数値が指定の範囲内か評価します。
     * 
     * @param n
     *            評価する数値
     * @param from
     *            範囲の最小値
     * @param to
     *            範囲の最大値
     * @return 指定の範囲内の場合のみ<code>true</code>
     */
    public static boolean isRange(long n, long from, long to) {
        if (from > to) {
            throw new IllegalArgumentException("from > to (" + from + " > " + to + ")");
        }
        return (from <= n && n <= to);
    }

    /**
     * 指定された数値が指定の範囲内か評価します。
     * 
     * @param n
     *            評価する数値
     * @param from
     *            範囲の最小値
     * @param to
     *            範囲の最大値
     * @return 指定の範囲内の場合のみ<code>true</code>
     */
    public static boolean isRange(double n, double from, double to) {
        if (from > to) {
            throw new IllegalArgumentException("from > to (" + from + " > " + to + ")");
        }
        return (from <= n && n <= to);
    }

    /*
     * Range Chars
     */

    /**
     * 指定された文字シーケンスの全要素が指定の範囲内か評価します。
     * 
     * @param cs
     *            評価する文字シーケンス
     * @param from
     *            範囲の最小値
     * @param to
     *            範囲の最大値
     * @return 文字シーケンスの全要素が指定の範囲内の場合のみ<code>true</code>
     */
    public static boolean isRangeChars(CharSequence cs, char from, char to) {
        if (cs == null) {
            return false;
        }
        int len = cs.length();
        for (int i = 0; i < len; i++) {
            if (!isRange(cs.charAt(i), from, to)) {
                return false;
            }
        }
        return true;
    }

    /*
     * Negative Number
     */

    /**
     * 指定された数値が負数か評価します。
     * 
     * @param n
     *            評価する数値
     * @return 指定された数値が負数の場合のみ<code>true</code>
     */
    public static boolean isNegative(int n) {
        return (0 > n);
    }

    /**
     * 指定された数値が負数か評価します。
     * 
     * @param n
     *            評価する数値
     * @return 指定された数値が負数の場合のみ<code>true</code>
     */
    public static boolean isNegative(long n) {
        return (0 > n);
    }

    /**
     * 指定された数値が負数か評価します。
     * 
     * @param n
     *            評価する数値
     * @return 指定された数値が負数の場合のみ<code>true</code>
     */
    public static boolean isNegative(double n) {
        return (0 > n);
    }

    /*
     * Positive Number
     */

    /**
     * 指定された数値が正数か評価します。
     * 
     * @param n
     *            評価する数値
     * @return 指定された数値が正数の場合のみ<code>true</code>
     */
    public static boolean isPositive(int n) {
        return (0 <= n);
    }

    /**
     * 指定された数値が正数か評価します。
     * 
     * @param n
     *            評価する数値
     * @return 指定された数値が正数の場合のみ<code>true</code>
     */
    public static boolean isPositive(long n) {
        return (0 <= n);
    }

    /**
     * 指定された数値が正数か評価します。
     * 
     * @param n
     *            評価する数値
     * @return 指定された数値が正数の場合のみ<code>true</code>
     */
    public static boolean isPositive(double n) {
        return (0 <= n);
    }

    /*
     * Min
     */

    /**
     * 指定された数値が指定の最小値の範囲か評価します。
     * 
     * @param n
     *            評価する数値
     * @param min
     *            範囲の最小値
     * @return 指定の最小値の範囲の場合のみ<code>true</code>
     */
    public static boolean isMin(int n, int min) {
        return (min <= n);
    }

    /**
     * 指定された数値が指定の最小値の範囲か評価します。
     * 
     * @param n
     *            評価する数値
     * @param min
     *            範囲の最小値
     * @return 指定の最小値の範囲の場合のみ<code>true</code>
     */
    public static boolean isMin(long n, long min) {
        return (min <= n);
    }

    /**
     * 指定された数値が指定の最小値の範囲か評価します。
     * 
     * @param n
     *            評価する数値
     * @param min
     *            範囲の最小値
     * @return 指定の最小値の範囲の場合のみ<code>true</code>
     */
    public static boolean isMin(double n, double min) {
        return (min <= n);
    }

    /*
     * Max
     */

    /**
     * 指定された数値が指定の最大値の範囲か評価します。
     * 
     * @param n
     *            評価する数値
     * @param max
     *            範囲の最大値
     * @return 指定の最大値の範囲の場合のみ<code>true</code>
     */
    public static boolean isMax(int n, int max) {
        return (n <= max);
    }

    /**
     * 指定された数値が指定の最大値の範囲か評価します。
     * 
     * @param n
     *            評価する数値
     * @param max
     *            範囲の最大値
     * @return 指定の最大値の範囲の場合のみ<code>true</code>
     */
    public static boolean isMax(long n, long max) {
        return (n <= max);
    }

    /**
     * 指定された数値が指定の最大値の範囲か評価します。
     * 
     * @param n
     *            評価する数値
     * @param max
     *            範囲の最大値
     * @return 指定の最大値の範囲の場合のみ<code>true</code>
     */
    public static boolean isMax(double n, double max) {
        return (n <= max);
    }

    /*
     * Contains
     */

    /**
     * 引数<code>e</code>が配列<code>a</code>に含まれているか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param e
     *            検索する値
     * @return 引数<code>e</code>が配列<code>a</code>に含まれている場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isContains(boolean[] a, boolean e) {
        for (int i = 0; i < a.length; i++) {
            if (e == a[i]) {
                return true;
            }
        }
        return false;
    }

    /**
     * 引数<code>e</code>が配列<code>a</code>に含まれているか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param e
     *            検索する値
     * @return 引数<code>e</code>が配列<code>a</code>に含まれている場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isContains(char[] a, char e) {
        for (int i = 0; i < a.length; i++) {
            if (e == a[i]) {
                return true;
            }
        }
        return false;
    }

    /**
     * 引数<code>e</code>が配列<code>a</code>に含まれているか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param e
     *            検索する値
     * @return 引数<code>e</code>が配列<code>a</code>に含まれている場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isContains(byte[] a, byte e) {
        for (int i = 0; i < a.length; i++) {
            if (e == a[i]) {
                return true;
            }
        }
        return false;
    }

    /**
     * 引数<code>e</code>が配列<code>a</code>に含まれているか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param e
     *            検索する値
     * @return 引数<code>e</code>が配列<code>a</code>に含まれている場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isContains(short[] a, short e) {
        for (int i = 0; i < a.length; i++) {
            if (e == a[i]) {
                return true;
            }
        }
        return false;
    }

    /**
     * 引数<code>e</code>が配列<code>a</code>に含まれているか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param e
     *            検索する値
     * @return 引数<code>e</code>が配列<code>a</code>に含まれている場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isContains(int[] a, int e) {
        for (int i = 0; i < a.length; i++) {
            if (e == a[i]) {
                return true;
            }
        }
        return false;
    }

    /**
     * 引数<code>e</code>が配列<code>a</code>に含まれているか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param e
     *            検索する値
     * @return 引数<code>e</code>が配列<code>a</code>に含まれている場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isContains(long[] a, long e) {
        for (int i = 0; i < a.length; i++) {
            if (e == a[i]) {
                return true;
            }
        }
        return false;
    }

    /**
     * 引数<code>e</code>が配列<code>a</code>に含まれているか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param e
     *            検索する値
     * @return 引数<code>e</code>が配列<code>a</code>に含まれている場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isContains(float[] a, float e) {
        for (int i = 0; i < a.length; i++) {
            if (e == a[i]) {
                return true;
            }
        }
        return false;
    }

    /**
     * 引数<code>e</code>が配列<code>a</code>に含まれているか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param e
     *            検索する値
     * @return 引数<code>e</code>が配列<code>a</code>に含まれている場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isContains(double[] a, double e) {
        for (int i = 0; i < a.length; i++) {
            if (e == a[i]) {
                return true;
            }
        }
        return false;
    }

    /**
     * 引数<code>o</code>が配列<code>a</code>に含まれているか評価します。
     * 
     * @param a
     *            評価基の配列
     * @param o
     *            検索するオブジェクト
     * @return 引数<code>o</code>が配列<code>a</code>に含まれている場合のみ<code>true</code>
     * @throws NullPointerException
     *             引数の配列が<code>null</code>の場合
     */
    public static boolean isContains(Object[] a, Object o) {
        for (int i = 0; i < a.length; i++) {
            if (isEquals(o, a[i])) {
                return true;
            }
        }
        return false;
    }

    /**
     * 引数<code>o</code>がコレクション<code>c</code>に含まれているか評価します。
     * 
     * @param c
     *            評価基のコレクション
     * @param o
     *            検索するオブジェクト
     * @return 引数<code>o</code>がコレクション<code>c</code>に含まれている場合のみ<code>true</code>
     */
    public static boolean isContains(Collection c, Object o) {
        if (c == null) {
            return false;
        }
        return c.contains(o);
    }

    /**
     * 指定の文字列に指定の文字が含まれているか評価します。
     * 
     * @param source
     *            評価する文字列
     * @param c
     *            検索する文字
     * @return 文字列に指定の文字が含まれている場合のみ<code>true</code>
     */
    public static boolean isContains(String source, char c) {
        if (source == null) {
            return false;
        }
        return (0 <= source.indexOf(c));
    }

    /**
     * 指定の文字列に指定の文字列が含まれているか評価します。
     * 
     * @param source
     *            評価する文字列
     * @param target
     *            検索する文字列
     * @return 文字列に指定の文字列が含まれている場合のみ<code>true</code>
     */
    public static boolean isContains(String source, String target) {
        if (source == null) {
            return false;
        }
        return (0 <= source.indexOf(target));
    }

    /**
     * 指定の文字シーケンスに指定の文字が含まれているか評価します。
     * 
     * @param cs
     *            評価する文字シーケンス
     * @param c
     *            検索する文字
     * @return 文字シーケンスに指定の文字が含まれている場合のみ<code>true</code>
     */
    public static boolean isContains(CharSequence cs, char c) {
        if (cs == null) {
            return false;
        }
        return (0 <= Seek.indexOf(cs, c));
    }

    /**
     * 指定の文字シーケンスに指定の文字シーケンスが含まれているか評価します。
     * 
     * @param cs
     *            評価する文字シーケンス
     * @param target
     *            検索する文字シーケンス
     * @return 文字シーケンスに指定の文字シーケンスが含まれている場合のみ<code>true</code>
     */
    public static boolean isContains(CharSequence cs, CharSequence target) {
        if (cs == null) {
            return false;
        }
        return (0 <= Seek.indexOf(cs, target));
    }

    /**
     * 指定のマップに指定の識別子が含まれているか評価します。
     * 
     * @param m
     *            評価するマップ
     * @param key
     *            識別子
     * @return マップに指定の識別子が含まれている場合のみ<code>true</code>
     */
    public static boolean isContainsKey(Map m, Object key) {
        return (m != null && m.containsKey(key));
    }

    /**
     * 指定のグループに指定の識別子が含まれているか評価します。
     * 
     * @param g
     *            評価するグループ
     * @param key
     *            識別子
     * @return グループに指定の識別子が含まれている場合のみ<code>true</code>
     */
    public static boolean isContainsKey(Group g, Object key) {
        return (g != null && g.containsKey(key));
    }

    /**
     * 指定のパラメータリストに指定の識別子が含まれているか評価します。
     * 
     * @param p
     *            評価するパラメータリスト
     * @param key
     *            識別子
     * @return パラメータリストに指定の識別子が含まれている場合のみ<code>true</code>
     */
    public static boolean isContainsKey(Parameters p, String key) {
        return (p != null && p.containsName(key));
    }

    /*
     * Type
     */

    /**
     * 指定されたオブジェクトが全てのクラス型に割り当て可能か評価します。
     * 
     * @param o
     *            評価するオブジェクト
     * @param class1
     *            割り当て可能か評価先するクラス
     * @param class2
     *            割り当て可能か評価先するクラス
     * @return オブジェクトが全てのクラス型に割り当て可能の場合のみ<code>true</code>
     */
    public static boolean isAndInstance(Object o, Class class1, Class class2) {
        return (class1.isInstance(o) && class2.isInstance(o));
    }

    /**
     * 指定されたオブジェクトが全てのクラス型に割り当て可能か評価します。
     * 
     * @param o
     *            評価するオブジェクト
     * @param classes
     *            割り当て可能か評価先するクラスの配列
     * @return オブジェクトが全てのクラス型に割り当て可能の場合のみ<code>true</code>
     */
    public static boolean isAndInstance(Object o, Class[] classes) {
        for (int i = 0; i < classes.length; i++) {
            Class c = classes[i];
            if (!c.isInstance(o)) {
                return false;
            }
        }
        return true;
    }

    /**
     * 指定されたオブジェクトがいずれかのクラス型に割り当て可能か評価します。
     * 
     * @param o
     *            評価するオブジェクト
     * @param class1
     *            割り当て可能か評価先するクラス
     * @param class2
     *            割り当て可能か評価先するクラス
     * @return オブジェクトがいずれかのクラス型に割り当て可能の場合のみ<code>true</code>
     */
    public static boolean isOrInstance(Object o, Class class1, Class class2) {
        return (class1.isInstance(o) || class2.isInstance(o));
    }

    /**
     * 指定されたオブジェクトがいずれかのクラス型に割り当て可能か評価します。
     * 
     * @param o
     *            評価するオブジェクト
     * @param classes
     *            割り当て可能か評価先するクラスの配列
     * @return オブジェクトがいずれかのクラス型に割り当て可能の場合のみ<code>true</code>
     */
    public static boolean isOrInstance(Object o, Class[] classes) {
        for (int i = 0; i < classes.length; i++) {
            Class c = classes[i];
            if (c.isInstance(o)) {
                return true;
            }
        }
        return false;
    }

    /**
     * クラスが同一であるかを評価します。
     * 
     * @param o
     *            評価するオブジェクト
     * @param c
     *            比較するクラス
     * @return クラスが同一の場合のみ<code>true</code>
     */
    public static boolean isType(Object o, Class c) {
        return (o != null && o.getClass().equals(c));
    }

    /**
     * 配列のコンポーネント型が指定のクラスと同一であるかを評価します。
     * 
     * @param a
     *            評価する配列
     * @param c
     *            評価先のクラス
     * @return 配列のコンポーネント型が指定のクラスと同一の場合のみ<code>true</code>
     */
    public static boolean isComponentType(Object a, Class c) {
        if (a == null) {
            return false;
        }
        Class type = a.getClass();
        if (!type.isArray()) {
            throw new IllegalArgumentException("is not Array");
        }
        return (type.getComponentType().equals(c));
    }

    /**
     * 配列型であるかを評価します。
     * 
     * @param o
     *            評価するオブジェクト
     * @return 配列型の場合のみ<code>true</code>
     */
    public static boolean isArray(Object o) {
        return (o != null && o.getClass().isArray());
    }

    /**
     * 配列型のクラスか評価します。
     * 
     * @param c
     *            評価するクラス
     * @return 配列型の場合のみ<code>true</code>
     */
    public static boolean isArrayClass(Class c) {
        return (c != null && c.isArray());
    }

    /*
     * index
     */

    /**
     * 指定されたインデックスが文字シーケンスの範囲内か評価します。
     * 
     * @param cs
     *            評価する文字シーケンス
     * @param index
     *            評価するインデックス
     * @return インデックスが文字シーケンスの範囲内の場合のみ<code>true</code>
     */
    public static boolean isIndex(CharSequence cs, int index) {
        if (cs == null) {
            return false;
        }
        return isRange(index, 0, (cs.length() - 1));
    }

    /**
     * 指定された文字シーケンスの範囲内のインデックスか評価します。
     * 
     * @param cs
     *            評価する文字シーケンス
     * @param min
     *            開始インデックス
     * @param max
     *            終了インデックス
     * @return 指定された文字シーケンスの範囲内のインデックスの場合のみ<code>true</code>
     */
    public static boolean isIndex(CharSequence cs, int min, int max) {
        if (cs == null) {
            return false;
        }
        if (min > max) {
            throw new IllegalArgumentException("min > max (" + min + " > " + max + ")");
        }
        int len = cs.length() - 1;
        return (isRange(min, 0, len) && isRange(max, 0, len));
    }

    /**
     * 指定されたインデックスがリストの範囲内か評価します。
     * 
     * @param list
     *            範囲を示すリスト
     * @param index
     *            評価するインデックス
     * @return インデックスがリストの範囲内の場合のみ<code>true</code>
     */
    public static boolean isIndex(List list, int index) {
        if (list == null) {
            return false;
        }
        return isRange(index, 0, (list.size() - 1));
    }

    /**
     * 指定されたリストの範囲内のインデックスか評価します。
     * 
     * @param list
     *            範囲を示すリスト
     * @param min
     *            開始インデックス
     * @param max
     *            終了インデックス
     * @return 指定されたリストの範囲内のインデックスの場合のみ<code>true</code>
     */
    public static boolean isIndex(List list, int min, int max) {
        if (list == null) {
            return false;
        }
        if (min > max) {
            throw new IllegalArgumentException("min > max (" + min + " > " + max + ")");
        }
        int size = list.size() - 1;
        return (isRange(min, 0, size) && isRange(max, 0, size));
    }

    /**
     * 指定されたインデックスが配列の範囲内か評価します。
     * 
     * @param a
     *            範囲を示す配列
     * @param index
     *            評価するインデックス
     * @return インデックスが配列の範囲内の場合のみ<code>true</code>
     * @throws IllegalArgumentException
     *             引数が配列でない場合
     */
    public static boolean isArrayIndex(Object a, int index) {
        if (a == null) {
            return false;
        }
        return isRange(index, 0, (Array.getLength(a) - 1));
    }

    /**
     * 指定された配列の範囲内のインデックスか評価します。
     * 
     * @param a
     *            評価する配列
     * @param min
     *            開始インデックス
     * @param max
     *            終了インデックス
     * @return 指定された配列の範囲内のインデックスの場合のみ<code>true</code>
     * @throws IllegalArgumentException
     *             引数が配列でない場合
     */
    public static boolean isArrayIndex(Object a, int min, int max) {
        if (a == null) {
            return false;
        }
        if (min > max) {
            throw new IllegalArgumentException("min > max (" + min + " > " + max + ")");
        }
        int size = Array.getLength(a) - 1;
        return (isRange(min, 0, size) && isRange(max, 0, size));
    }

    /*
     * size
     */

    /**
     * 指定された文字シーケンスの長さが指定の長さと同一か評価します。
     * 
     * @param a
     *            評価する文字シーケンス
     * @param length
     *            文字シーケンスの長さ
     * @return 指定された文字シーケンスの長さが指定の長さと同一の場合のみ<code>true</code>
     */
    public static boolean isSize(CharSequence a, int length) {
        if (a == null) {
            return false;
        }
        return (a.length() == length);
    }

    /**
     * 指定された文字シーケンスの長さが指定の長さの範囲内か評価します。
     * 
     * @param cs
     *            評価する文字シーケンス
     * @param min
     *            文字数の最小値
     * @param max
     *            文字数の最大値
     * @return 指定された文字シーケンスの長さが指定の長さの範囲内の場合のみ<code>true</code>
     */
    public static boolean isSize(CharSequence cs, int min, int max) {
        if (cs == null) {
            return false;
        }
        if (min > max) {
            throw new IllegalArgumentException("min > max (" + min + " > " + max + ")");
        }
        int size = cs.length();
        return (isRange(min, 0, size) && isRange(max, 0, size));
    }

    /**
     * 指定された配列の長さが指定の配列長と同一か評価します。
     * 
     * @param a
     *            評価する配列
     * @param length
     *            配列長
     * @return 指定された配列の長さが指定の配列長と同一の場合のみ<code>true</code>
     * @throws IllegalArgumentException
     *             引数が配列でない場合
     */
    public static boolean isArraySize(Object a, int length) {
        if (a == null) {
            return false;
        }
        return (Array.getLength(a) == length);
    }

    /**
     * 指定された配列の範囲内のインデックスか評価します。
     * 
     * @param a
     *            評価する配列
     * @param min
     *            文字数の最小値
     * @param max
     *            文字数の最大値
     * @return 指定された配列の範囲内のインデックスの場合のみ<code>true</code>
     * @throws IllegalArgumentException
     *             引数が配列でない場合
     */
    public static boolean isArraySize(Object a, int min, int max) {
        if (a == null) {
            return false;
        }
        if (min > max) {
            throw new IllegalArgumentException("min > max (" + min + " > " + max + ")");
        }
        int len = Array.getLength(a);
        return (isRange(len, min, max));
    }

    /**
     * 指定された全ての配列の長さが同一か評価します。
     * 
     * @param as
     *            評価するの配列を格納とする配列
     * @return 指定された全ての配列の長さが同一の場合のみ<code>true</code>
     * @throws IllegalArgumentException
     *             引数の要素が配列でないまたは引数が空配列の場合
     */
    public static boolean isAndArraySize(Object[] as) {
        if (isEmpty(as)) {
            throw new IllegalArgumentException("array as empty");
        }
        int len = Array.getLength(as[0]);
        for (int i = 1; i < as.length; i++) {
            if (len != Array.getLength(as[i])) {
                return false;
            }
        }
        return true;
    }

    /*
     * charAt
     */

    /**
     * 指定されたインデックスの文字が指定の文字と同一か評価します。 <br>
     * 文字シーケンスが<code>null</code>の場合と不正なインデックスの場合は<code>false</code>を返却します。
     * 
     * @param cs
     *            評価する文字シーケンス
     * @param index
     *            評価する文字のインデックス
     * @param c
     *            比較先の文字
     * @return 指定されたインデックスの文字が指定の文字と同一の場合のみ<code>true</code>
     */
    public static boolean isCharAt(CharSequence cs, int index, char c) {
        return (isIndex(cs, index) && cs.charAt(index) == c);
    }

    /**
     * 指定されたインデックスの文字が指定の文字の内の何れかと同一か評価します。 <br>
     * 文字シーケンスが<code>null</code>の場合と不正なインデックスの場合は<code>false</code>を返却します。
     * 
     * @param cs
     *            評価する文字シーケンス
     * @param index
     *            評価する文字のインデックス
     * @param ac
     *            比較先の文字の配列
     * @return 指定されたインデックスの文字が指定の文字の内の何れかと同一の場合のみ<code>true</code>
     */
    public static boolean isOrCharAt(CharSequence cs, int index, char[] ac) {
        if (isIndex(cs, index)) {
            char c = cs.charAt(index);
            for (int i = 0; i < ac.length; i++) {
                if (c == ac[i]) {
                    return true;
                }
            }
        }
        return false;
    }

}
