/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.helpers;

import java.lang.reflect.Array;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * オブジェクトの内部情報を探索するヘルパーメソッド群を提供します。
 */
public class Seek {

    /**
     * 引数に一致する文字をスキップしたインデックスを返却します。
     * 
     * @param cs
     *            文字シーケンス
     * @param c
     *            スキップ対象の文字
     * @param fromIndex
     *            探索の開始位置
     * @return 引数に一致する文字をスキップしたインデックス
     */
    public static int skip(CharSequence cs, char c, int fromIndex) {
        int inx = fromIndex;
        while (inx < cs.length()) {
            if (cs.charAt(inx) != c) {
                break;
            }
            inx++;
        }
        return inx;
    }

    /**
     * 空白文字をスキップしたインデックスを返却します。
     * 
     * @param cs
     *            文字シーケンス
     * @param fromIndex
     *            探索の開始位置
     * @return 空白文字をスキップしたインデックス
     */
    public static int skipSp(CharSequence cs, int fromIndex) {
        int inx = fromIndex;
        while (inx < cs.length()) {
            if (cs.charAt(inx) > ' ') {
                break;
            }
            inx++;
        }
        return inx;
    }

    /**
     * 引数に一致する文字をスキップしたインデックスを返却します。
     * 
     * @param cs
     *            文字シーケンス
     * @param chars
     *            スキップ対象の文字の配列
     * @param fromIndex
     *            探索の開始位置
     * @return 引数に一致する文字をスキップしたインデックス
     */
    public static int skip(CharSequence cs, char[] chars, int fromIndex) {
        int inx = fromIndex;
        while (inx < cs.length()) {
            char c = cs.charAt(inx);
            boolean hit = false;
            for (int i = 0; i < chars.length; i++) {
                if (c == chars[i]) {
                    hit = true;
                    break;
                }
            }
            if (!hit) {
                break;
            }
            inx++;
        }
        return inx;
    }

    /**
     * 指定の値と一致するコレクションの要素のカウント数を返却します。
     * 
     * @param c
     *            走査対象のコレクション
     * @param o
     *            検索する値
     * @return 指定の値と一致するコレクションの要素のカウント数
     */
    public static int count(Collection c, Object o) {
        if (c instanceof Set) {
            return (c.contains(o)) ? 1 : 0;
        } else {
            int count = 0;
            for (Iterator i = c.iterator(); i.hasNext();) {
                if (Eval.isEquals(i.next(), o)) {
                    count++;
                }
            }
            return count;
        }
    }

    // /**
    // * 指定の値と一致するコレクションの全ての要素をコレクションに格納して返却します。
    // *
    // * @param c
    // * 走査対象のコレクション
    // * @param o
    // * 検索する値
    // * @return 指定の値と一致するコレクションの全ての要素
    // */
    // public static Collection getAll(Collection c, Object o) {
    // List a = new ArrayList();
    // for (Iterator i = c.iterator(); i.hasNext();) {
    // Object e = i.next();
    // if (Eval.isEquals(e, o)) {
    // a.add(e);
    // }
    // }
    // return a;
    // }

    /**
     * 文字シーケンス内の指定の文字と一致する文字を検索してそのインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param c
     *            検索する文字
     * @return 発見出来た場合そのインデックス、以外は -1
     */
    public static int indexOf(CharSequence cs, char c) {
        return indexOf(cs, c, 0, (cs.length() - 1));
    }

    /**
     * 文字シーケンス内の指定の文字と一致する文字を指定の開始位置から検索してそのインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param c
     *            検索する文字
     * @param fromIndex
     *            検索の開始位置を示すインデックス
     * @return 発見出来た場合そのインデックス、以外は -1
     */
    public static int indexOf(CharSequence cs, char c, int fromIndex) {
        return indexOf(cs, c, fromIndex, (cs.length() - 1));
    }

    /**
     * 文字シーケンス内の指定の文字と一致する文字を指定の開始位置と終了位置の間を検索してそのインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param c
     *            検索する文字
     * @param fromIndex
     *            検索の開始位置を示すインデックス
     * @param toIndex
     *            検索の終了位置を示すインデックス
     * @return 発見出来た場合そのインデックス、以外は -1
     */
    public static int indexOf(CharSequence cs, char c, int fromIndex, int toIndex) {
        if (!Eval.isIndex(cs, fromIndex, toIndex)) {
            throw new IllegalArgumentException("no index. length:" + cs.length() + ", fromIndex:" + fromIndex
                    + ", toIndex:" + toIndex);
        }
        for (int i = fromIndex; i <= toIndex; i++) {
            if (cs.charAt(i) == c) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 文字シーケンス内の指定の文字と一致する文字を後方検索してそのインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param c
     *            検索する文字
     * @return 発見出来た場合そのインデックス、以外は -1
     */
    public static int lastIndexOf(CharSequence cs, char c) {
        return lastIndexOf(cs, c, 0, (cs.length() - 1));
    }

    /**
     * 文字シーケンス内の指定の文字と一致する文字を指定の開始位置から後方検索してそのインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param c
     *            検索する文字
     * @param fromIndex
     *            検索の開始位置を示すインデックス
     * @return 発見出来た場合そのインデックス、以外は -1
     */
    public static int lastIndexOf(CharSequence cs, char c, int fromIndex) {
        return lastIndexOf(cs, c, fromIndex, (cs.length() - 1));
    }

    /**
     * 文字シーケンス内の指定の文字と一致する文字を指定の開始位置と終了位置の間を後方検索してそのインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param c
     *            検索する文字
     * @param fromIndex
     *            検索の開始位置を示すインデックス
     * @param toIndex
     *            検索の終了位置を示すインデックス
     * @return 発見出来た場合そのインデックス、以外は -1
     */
    public static int lastIndexOf(CharSequence cs, char c, int fromIndex, int toIndex) {
        if (!Eval.isIndex(cs, fromIndex, toIndex)) {
            throw new IllegalArgumentException("no index. length:" + cs.length() + ", fromIndex:" + fromIndex
                    + ", toIndex:" + toIndex);
        }
        for (int i = toIndex; i >= fromIndex; i--) {
            if (cs.charAt(i) == c) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 文字シーケンス内の指定の文字の何れかと一致する文字を検索してそのインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param c
     *            検索する文字
     * @return 発見出来た場合そのインデックス、以外は -1
     */
    public static int orIndexOf(CharSequence cs, char[] c) {
        return orIndexOf(cs, c, 0, (cs.length() - 1));
    }

    /**
     * 文字シーケンス内の指定の文字の何れかと一致する文字を指定の開始位置から検索してそのインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param c
     *            検索する文字
     * @param fromIndex
     *            検索の開始位置を示すインデックス
     * @return 発見出来た場合そのインデックス、以外は -1
     */
    public static int orIndexOf(CharSequence cs, char[] c, int fromIndex) {
        return orIndexOf(cs, c, fromIndex, (cs.length() - 1));
    }

    /**
     * 文字シーケンス内の指定の文字の何れかと一致する文字を指定の開始位置と終了位置の間を検索してそのインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param c
     *            検索する文字
     * @param fromIndex
     *            検索の開始位置を示すインデックス
     * @param toIndex
     *            検索の終了位置を示すインデックス
     * @return 発見出来た場合そのインデックス、以外は -1
     */
    public static int orIndexOf(CharSequence cs, char[] c, int fromIndex, int toIndex) {
        if (!Eval.isIndex(cs, fromIndex, toIndex)) {
            throw new IllegalArgumentException("no index. length:" + cs.length() + ", fromIndex:" + fromIndex
                    + ", toIndex:" + toIndex);
        }
        for (int i = fromIndex; i <= toIndex; i++) {
            for (int j = 0; j < c.length; j++) {
                if (cs.charAt(i) == c[j]) {
                    return i;
                }
            }
        }
        return -1;
    }

    /**
     * 文字シーケンス内の指定の文字の何れかと一致する文字を後方検索してそのインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param c
     *            検索する文字
     * @return 発見出来た場合そのインデックス、以外は -1
     */
    public static int orLastIndexOf(CharSequence cs, char[] c) {
        return orLastIndexOf(cs, c, 0, (cs.length() - 1));
    }

    /**
     * 文字シーケンス内の指定の文字の何れかと一致する文字を指定の開始位置から後方検索してそのインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param c
     *            検索する文字
     * @param fromIndex
     *            検索の開始位置を示すインデックス
     * @return 発見出来た場合そのインデックス、以外は -1
     */
    public static int orLastIndexOf(CharSequence cs, char[] c, int fromIndex) {
        return orLastIndexOf(cs, c, fromIndex, (cs.length() - 1));
    }

    /**
     * 文字シーケンス内の指定の文字の何れかと一致する文字を指定の開始位置と終了位置の間を後方検索してそのインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param c
     *            検索する文字
     * @param fromIndex
     *            検索の開始位置を示すインデックス
     * @param toIndex
     *            検索の終了位置を示すインデックス
     * @return 発見出来た場合そのインデックス、以外は -1
     */
    public static int orLastIndexOf(CharSequence cs, char[] c, int fromIndex, int toIndex) {
        if (!Eval.isIndex(cs, fromIndex, toIndex)) {
            throw new IllegalArgumentException("no index. length:" + cs.length() + ", fromIndex:" + fromIndex
                    + ", toIndex:" + toIndex);
        }
        for (int i = toIndex; i >= fromIndex; i--) {
            for (int j = 0; j < c.length; j++) {
                if (cs.charAt(i) == c[j]) {
                    return i;
                }
            }
        }
        return -1;
    }

    /**
     * 文字シーケンス内の指定の文字と一致する文字シーケンスを検索してその開始位置のインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param target
     *            検索する文字シーケンス
     * @return 発見出来た場合その開始位置のインデックス、以外は -1
     */
    public static int indexOf(CharSequence cs, CharSequence target) {
        return indexOf(cs, 0, (cs.length() - 1), target);
    }

    /**
     * 文字シーケンス内の指定の文字と一致する文字シーケンスを指定の開始位置から検索してその開始位置のインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param target
     *            検索する文字シーケンス
     * @param fromIndex
     *            検索の開始位置を示すインデックス
     * @return 発見出来た場合その開始位置のインデックス、以外は -1
     */
    public static int indexOf(CharSequence cs, int fromIndex, CharSequence target) {
        return indexOf(cs, fromIndex, (cs.length() - 1), target);
    }

    /**
     * 文字シーケンス内の指定の文字と一致する文字シーケンスを指定の開始位置と終了位置の間を検索してその開始位置のインデックスを返します。<br>
     * 発見出来なかった場合は -1 を返します。
     * 
     * @param cs
     *            検索対象の文字シーケンス
     * @param target
     *            検索する文字シーケンス
     * @param fromIndex
     *            検索の開始位置を示すインデックス
     * @param toIndex
     *            検索の終了位置を示すインデックス
     * @return 発見出来た場合その開始位置のインデックス、以外は -1
     */
    public static int indexOf(CharSequence cs, int fromIndex, int toIndex, CharSequence target) {
        if (!Eval.isIndex(cs, fromIndex, toIndex)) {
            throw new IllegalArgumentException("no index. length:" + cs.length() + ", fromIndex:" + fromIndex
                    + ", toIndex:" + toIndex);
        }
        // target Blank As Not Found.
        if (!Eval.isBlank(target)) {
            return -1;
        }

        char first = target.charAt(0);
        for (int i = fromIndex; i <= toIndex; i++) {
            // Look for first character.
            if (cs.charAt(i) != first) {
                while (++i <= toIndex && cs.charAt(i) != first) {
                    // no op
                }
            }
            // Look for next's character.
            int targetCount = target.length();
            if (i <= toIndex) {
                int j = i + 1;
                int max = j + targetCount - 1;
                for (int k = 1; j < max && cs.charAt(j) == target.charAt(k); j++, k++) {
                    // no op
                }
                if (j == max) {
                    // Found whole string.
                    return i;
                }
            }
        }
        // Not Found.
        return -1;
    }

    /**
     * 指定された値と同値の要素を開始位置から終了位置まで線形検索しそのインデックスを返却します。 <br>
     * <code>o</code>が<code>null</code>の場合<code>(a[i] == null)</code>で検証します。
     * 
     * @param list
     *            検索元のリスト
     * @param o
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置がリストの範囲外の場合
     */
    public static int indexOf(List list, Object o, int fromIndex, int toIndex) {
        checkIndexOfException(list, fromIndex, toIndex);
        if (o == null) {
            for (int i = fromIndex; i <= toIndex; i++) {
                if (list.get(i) == null) {
                    return i;
                }
            }
        } else {
            for (int i = fromIndex; i <= toIndex; i++) {
                if (o.equals(list.get(i))) {
                    return i;
                }
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を終了位置から開始位置まで線形検索しそのインデックスを返却します。 <br>
     * <code>o</code>が<code>null</code>の場合<code>(a[i] == null)</code>で検証します。
     * 
     * @param list
     *            検索元のリスト
     * @param o
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置がリストの範囲外の場合
     */
    public static int lastIndexOf(List list, Object o, int fromIndex, int toIndex) {
        checkIndexOfException(list, fromIndex, toIndex);
        if (o == null) {
            for (int i = toIndex; i >= fromIndex; i--) {
                if (list.get(i) == null) {
                    return i;
                }
            }
        } else {
            for (int i = toIndex; i >= fromIndex; i--) {
                if (o.equals(list.get(i))) {
                    return i;
                }
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を開始位置から終了位置まで線形検索しそのインデックスを返却します。 <br>
     * <code>o</code>が<code>null</code>の場合<code>(a[i] == null)</code>で検証します。
     * 
     * @param a
     *            検索元の配列
     * @param o
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int indexOf(Object[] a, Object o, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        if (o == null) {
            for (int i = fromIndex; i <= toIndex; i++) {
                if (a[i] == null) {
                    return i;
                }
            }
        } else {
            for (int i = fromIndex; i <= toIndex; i++) {
                if (o.equals(a[i])) {
                    return i;
                }
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を終了位置から開始位置まで線形検索しそのインデックスを返却します。 <br>
     * <code>o</code>が<code>null</code>の場合<code>(a[i] == null)</code>で検証します。
     * 
     * @param a
     *            検索元の配列
     * @param o
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int lastIndexOf(Object[] a, Object o, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        if (o == null) {
            for (int i = toIndex; i >= fromIndex; i--) {
                if (a[i] == null) {
                    return i;
                }
            }
        } else {
            for (int i = toIndex; i >= fromIndex; i--) {
                if (o.equals(a[i])) {
                    return i;
                }
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を開始位置から終了位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int indexOf(boolean[] a, boolean e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = fromIndex; i <= toIndex; i++) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を終了位置から開始位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int lastIndexOf(boolean[] a, boolean e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = toIndex; i >= fromIndex; i--) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を開始位置から終了位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int indexOf(char[] a, char e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = fromIndex; i <= toIndex; i++) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を終了位置から開始位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int lastIndexOf(char[] a, char e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = toIndex; i >= fromIndex; i--) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を開始位置から終了位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int indexOf(byte[] a, byte e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = fromIndex; i <= toIndex; i++) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を終了位置から開始位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int lastIndexOf(byte[] a, byte e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = toIndex; i >= fromIndex; i--) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を開始位置から終了位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int indexOf(short[] a, short e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = fromIndex; i <= toIndex; i++) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を終了位置から開始位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int lastIndexOf(short[] a, short e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = toIndex; i >= fromIndex; i--) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を開始位置から終了位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int indexOf(int[] a, int e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = fromIndex; i <= toIndex; i++) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を終了位置から開始位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int lastIndexOf(int[] a, int e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = toIndex; i >= fromIndex; i--) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を開始位置から終了位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int indexOf(long[] a, long e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = fromIndex; i <= toIndex; i++) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を終了位置から開始位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int lastIndexOf(long[] a, long e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = toIndex; i >= fromIndex; i--) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を開始位置から終了位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int indexOf(float[] a, float e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = fromIndex; i <= toIndex; i++) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を終了位置から開始位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int lastIndexOf(float[] a, float e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = toIndex; i >= fromIndex; i--) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を開始位置から終了位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int indexOf(double[] a, double e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = fromIndex; i <= toIndex; i++) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 指定された値と同値の要素を終了位置から開始位置まで線形検索しそのインデックスを返却します。
     * 
     * @param a
     *            検索元の配列
     * @param e
     *            検索する値
     * @param fromIndex
     *            検索の開始インデックス
     * @param toIndex
     *            検索の終了インデックス
     * @return 同値の要素が最初に検出されたインデックス、同値が無い場合は -1
     * @throws NullPointerException
     *             <code>a</code>に<code>null</code>が指定された場合
     * @throws IndexOutOfBoundsException
     *             開始位置または終了位置が配列の範囲外の場合
     */
    public static int lastIndexOf(double[] a, double e, int fromIndex, int toIndex) {
        checkIndexOfException(a, fromIndex, toIndex);
        for (int i = toIndex; i >= fromIndex; i--) {
            if (e == a[i]) {
                return i;
            }
        }
        return -1;
    }

    /* リストを検索するメソッドの引数を検証する機能。 */
    private static void checkIndexOfException(List list, int fromIndex, int toIndex) {
        if (list == null) {
            throw new NullPointerException("list is null.");
        }
        if (!Eval.isIndex(list, fromIndex, toIndex)) {
            throw new IndexOutOfBoundsException("fromIndex:" + fromIndex + ", toIndex:" + toIndex + ", size:"
                    + list.size());
        }
    }

    /* 配列を検索するメソッドの引数を検証する機能。 */
    private static void checkIndexOfException(Object a, int fromIndex, int toIndex) {
        if (a == null) {
            throw new NullPointerException("a is null.");
        }
        if (!Eval.isArray(a)) {
            throw new IllegalArgumentException("is not Array:" + a);
        }
        if (!Eval.isArrayIndex(a, fromIndex, toIndex)) {
            throw new IndexOutOfBoundsException("fromIndex:" + fromIndex + ", toIndex:" + toIndex + ", size:"
                    + Array.getLength(a));
        }
    }

}
