/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.core.lang;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * 基本的な言語機能のユーティリティメソッドを提供します。
 */
public class LangUtils {

    /**
     * １次元または多次元配列の基に為るクラス型を取得します。配列以外の場合は指定のクラスがそのまま返ります。 <br>
     * 例：<code>java.lang.String[][]</code>の配列型に対しては<code>java.lang.String</code>クラスが返されます。
     * 
     * @param c
     *            検証するクラス
     * @return １次元または多次元配列の基に為るクラス型
     * @throws NullPointerException
     *             引数に<code>null</code>が指定された場合
     */
    public static Class getSourceComponentType(Class c) {
        if (c.isArray()) {
            return getSourceComponentType(c.getComponentType());
        }
        return c;
    }

    /** プリミティブ型のクラスと省略名のマッピング。 */
    public static final Map PRIMITIVE_FOR_TYPE_MAP;
    static {
        HashMap m = new HashMap();
        // primitive
        m.put("byte", Byte.TYPE);
        m.put("short", Short.TYPE);
        m.put("int", Integer.TYPE);
        m.put("long", Long.TYPE);
        m.put("float", Float.TYPE);
        m.put("double", Double.TYPE);
        m.put("boolean", Boolean.TYPE);
        m.put("char", Character.TYPE);
        m.put("void", Void.TYPE);
        PRIMITIVE_FOR_TYPE_MAP = Collections.unmodifiableMap(m);
    }

    /** 基本的な型のクラスと省略名のマッピング。 */
    public static final Map SHORT_NAME_FOR_TYPE_MAP;
    static {
        HashMap m = new HashMap();
        // primitive
        m.putAll(PRIMITIVE_FOR_TYPE_MAP);
        // primitive object
        m.put("Byte", Byte.class);
        m.put("Short", Short.class);
        m.put("Integer", Integer.class);
        m.put("Long", Long.class);
        m.put("Float", Float.class);
        m.put("Double", Double.class);
        m.put("Boolean", Boolean.class);
        m.put("Character", Character.class);
        m.put("Void", Void.class);
        // object
        m.put("Object", Object.class);
        m.put("Class", Class.class);
        m.put("System", System.class);
        m.put("String", String.class);
        SHORT_NAME_FOR_TYPE_MAP = Collections.unmodifiableMap(m);
    }

    /** プリミティブ型のクラスとクラス名のマッピング。 */
    public static final Map PRIMITIVE_CLASS_NAME_MAP;
    static {
        HashMap m = new HashMap();
        // primitive
        m.put(Byte.TYPE, "B");
        m.put(Short.TYPE, "S");
        m.put(Integer.TYPE, "I");
        m.put(Long.TYPE, "J");
        m.put(Float.TYPE, "F");
        m.put(Double.TYPE, "D");
        m.put(Boolean.TYPE, "Z");
        m.put(Character.TYPE, "C");
        m.put(Void.TYPE, "V");
        PRIMITIVE_CLASS_NAME_MAP = Collections.unmodifiableMap(m);
    }

    /**
     * 拡張されたクラスの表記文字列からクラスをリードし返却します。
     * 
     * @param name
     *            クラスを示す文字列
     * @param loader
     *            クラスローダー
     * @return ロードされたクラス
     * @throws NoSuchResourceException
     *             クラスのリードに失敗した場合
     */
    public static Class loadClass(String name, ClassLoader loader) throws NoSuchResourceException {

        // 拡張の配列表記の検証 ”String[][] 等”
        int offset = name.length();
        StringBuffer aSymbol = new StringBuffer();
        if (offset >= 2) {
            while (name.charAt(offset - 2) == '[' && name.charAt(offset - 1) == ']') {
                offset -= 2;
                aSymbol.append('[');
            }
        }
        // 配列型=TRUE
        boolean isArray = (aSymbol.length() > 0);

        // クラス名を取得（配列の場合は基クラス）
        String className = (isArray) ? name.substring(0, offset) : name;
        // 拡張のクラス表記の検証
        Class c = (Class) SHORT_NAME_FOR_TYPE_MAP.get(className);
        // 配列の場合は配列の表記を付加する
        if (isArray) {
            // 拡張のクラス表記が検出された場合プリミティブ型の検証をする。
            if (c != null) {
                String primitive = (String) PRIMITIVE_CLASS_NAME_MAP.get(c);
                if (primitive == null) {
                    className = aSymbol + "L" + c.getName() + ";";
                } else {
                    className = aSymbol + primitive;
                }
            } else {
                className = aSymbol + "L" + className + ";";
            }
        } else {
            // 配列型以外の拡張のクラス表記は返却
            if (c != null) {
                return c;
            }
        }

        // クラスリード（拡張表記以外は変更なし）
        return ResourceLoader.getClass(className, loader);

    }

}
