/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.core.util;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Iterator;

import shohaku.core.helpers.Cutter;
import shohaku.core.lang.NoSuchResourceException;
import shohaku.core.lang.ResourceLoader;
import shohaku.core.util.cel.CELBinder;
import shohaku.core.util.cel.CELContext;

/**
 * プロパティ値を<code>CEL(Composite CELExpression Language)</code>文として認識するプロパティリストを提供します。 <br>
 * <code>CEL</code>がシリアライズ機能を提供しないため、このプロパティリストもシリアライズ機能を提供していません。
 */
public class CELProperties extends AbstractXProperties {

    /** 使用する式言語バインダー。 */
    protected final CELBinder binder;

    /** CELコンテキスト、'$'で始まるプロパティを要素として保管する。 */
    protected final CELContext context = new CELContext() {
        private HashMap m = new HashMap();

        public Iterator nameIterator() {
            return m.keySet().iterator();
        }

        public Object getValue(String name) {
            return m.get(name);
        }

        public Object setValue(String name, Object value) {
            return m.put(name, value);
        }

        public boolean containsName(String name) {
            return m.containsKey(name);
        }

    };

    /**
     * デフォルトの式言語バインダーで初期化します。 <br>
     * <code>CELBinder.getObjectCreationBinder()</code>をデフォルトバインダーとしています。
     */
    public CELProperties() {
        this(CELBinder.getObjectCreationBinder());
    }

    /**
     * 使用する式言語バインダーを指定して初期化します。
     * 
     * @param binder
     *            式言語バインダー
     */
    public CELProperties(CELBinder binder) {
        super();
        this.binder = binder;
    }

    /**
     * 入力ストリームからキーと要素が対になったプロパティリストを読み込みます。 <br>
     * ストリームは<code>ISO 8859-1</code>文字エンコーディングを使用しているとみなされます。 <br>
     * このエンコーディングに直接表示できない文字には<code>Unicode escapes</code>が使用されます。 <br>
     * ただし、エスケープシーケンスでは 1 文字の「u」だけが使用可能です。 <br>
     * 他の文字エンコーディングとプロパティファイルを変換する場合<code>native2ascii</code>ツールを使用できます。
     * 
     * @param inStream
     *            入力ストリーム
     * @throws IOException
     *             入力ストリームからの読み込み中にエラーが発生した場合
     */
    public synchronized void load(InputStream inStream) throws IOException {
        loadImpl(inStream, DEFAULT_CHARSET, true);
    }

    /**
     * 入力ストリームからキーと要素が対になったプロパティリストを読み込みます。 <br>
     * ストリームは、引数 charsetName で指定された文字エンコーディングを使用しているとみなされます。 <br>
     * このメソッドは以下と等価です。 <br>
     * <code>load(in, Charset.forName(charsetName));</code>
     * 
     * @param inStream
     *            入力ストリーム
     * @param charsetName
     *            文字エンコーディング
     * @throws IOException
     *             入力ストリームからの読み込み中にエラーが発生した場合
     */
    public synchronized void load(InputStream inStream, String charsetName) throws IOException {
        loadImpl(inStream, Charset.forName(charsetName), false);
    }

    /**
     * 入力ストリームからキーと要素が対になったプロパティリストを読み込みます。 <br>
     * ストリームは、引数 charset で指定された文字エンコーディングを使用しているとみなされます。
     * 
     * @param inStream
     *            入力ストリーム
     * @param charset
     *            文字エンコーディング
     * @throws IOException
     *             入力ストリームからの読み込み中にエラーが発生した場合
     */
    public synchronized void load(InputStream inStream, Charset charset) throws IOException {
        loadImpl(inStream, charset, false);
    }

    /**
     * 指定されたキーを持つプロパティを、プロパティリストから探します。 <br>
     * キーが存在しない場合は<code>null</code>が返されます。
     * 
     * @param key
     *            プロパティキー
     * @return 指定されたキー値を持つこのプロパティリストの値
     */
    public Object getProperty(String key) {
        return lookup.get(key);
    }

    /**
     * 指定されたキーを持つプロパティを、プロパティリストから探します。 <br>
     * キーが存在しない場合は<code>defaultValue</code>が返されます。
     * 
     * @param key
     *            プロパティキー
     * @param defaultValue
     *            デフォルト値
     * @return 指定されたキー値を持つこのプロパティリストの値
     */
    public Object getProperty(String key, Object defaultValue) {
        Object o = getProperty(key);
        return (o == null) ? defaultValue : o;
    }

    /**
     * プロパティを登録します。
     * 
     * @param key
     *            プロパティキー
     * @param value
     *            プロパティ値
     * @see shohaku.core.util.AbstractXProperties#putProperty(java.lang.String, java.lang.String)
     */
    protected void putProperty(String key, String value) throws IOException {
        if (key.charAt(0) == '$') {
            context.setValue(key.substring(1), binder.getValue(value, context));
        } else if (key.charAt(0) == '@') {
            if (key.equals("@import")) {
                Class c = null;
                try {
                    c = ResourceLoader.getClass(value);
                } catch (NoSuchResourceException e) {
                    throw new IOException(e.getMessage());
                }
                String className = Cutter.cutLastIndexOf(c.getName(), '.', 0);
                context.setValue(className, c);
            }
        } else {
            lookup.put(key, binder.getValue(value, context));
        }

    }

}
