/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.core.util;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.ParseException;
import java.util.Locale;

/**
 * 数値型のユーティリティメソッドを提供します。
 */
public class DecimalUtils {

    /*
     * parse
     */

    /**
     * 文字列を指定された書式とデフォルトロケールで数値型に変換します、変換に失敗した場合<code>null</code>を返却します。
     * 
     * @param text
     *            文字列
     * @param pattern
     *            書式
     * @return 変換後の文字列
     */
    public static Number parse(String text, String pattern) {
        return parse(Locale.getDefault(), text, pattern);
    }

    /**
     * 文字列を指定された書式と指定のロケールで数値型に変換します、変換に失敗した場合<code>null</code>を返却します。
     * 
     * @param locale
     *            ロケール
     * @param text
     *            文字列
     * @param pattern
     *            書式
     * @return 変換後の文字列
     */
    public static Number parse(Locale locale, String text, String pattern) {
        return parse(new DecimalFormatSymbols(locale), text, pattern);
    }

    /**
     * 文字列を指定された書式と指定の数値変換の記号セットで数値型に変換します、変換に失敗した場合<code>null</code>を返却します。
     * 
     * @param symbols
     *            数値変換の記号セット
     * @param text
     *            文字列
     * @param pattern
     *            書式
     * @return 変換後の文字列
     */
    public static Number parse(DecimalFormatSymbols symbols, String text, String pattern) {
        try {
            DecimalFormat format = new DecimalFormat(pattern, symbols);
            return format.parse(text);
        } catch (ParseException ex) {
            return null;
        }
    }

    /*
     * format
     */

    /**
     * 数値(long)を引数で指定される書式とデフォルトロケールで文字列に変換します。
     * 
     * @param number
     *            数値
     * @param pattern
     *            書式
     * @return 変換後の文字列
     */
    public static String format(long number, String pattern) {
        return format(Locale.getDefault(), number, pattern);
    }

    /**
     * 数値(long)を引数で指定される書式と指定のロケールで文字列に変換します。
     * 
     * @param locale
     *            ロケール
     * @param number
     *            数値
     * @param pattern
     *            書式
     * @return 変換後の文字列
     */
    public static String format(Locale locale, long number, String pattern) {
        DecimalFormat format = new DecimalFormat(pattern, new DecimalFormatSymbols(locale));
        return format.format(number);
    }

    /**
     * 数値(long)を引数で指定される書式と指定の数値変換の記号セットで文字列に変換します。
     * 
     * @param symbols
     *            数値変換の記号セット
     * @param number
     *            数値
     * @param pattern
     *            書式
     * @return 変換後の文字列
     */
    public static String format(DecimalFormatSymbols symbols, long number, String pattern) {
        DecimalFormat format = new DecimalFormat(pattern, symbols);
        return format.format(number);
    }

    /**
     * 数値(double)を引数で指定される書式とデフォルトロケールで文字列に変換します。
     * 
     * @param number
     *            数値
     * @param pattern
     *            書式
     * @return 変換後の文字列
     */
    public static String format(double number, String pattern) {
        return format(number, pattern);
    }

    /**
     * 数値(double)を引数で指定される書式と指定のロケールで文字列に変換します。
     * 
     * @param locale
     *            ロケール
     * @param number
     *            数値
     * @param pattern
     *            書式
     * @return 変換後の文字列
     */
    public static String format(Locale locale, double number, String pattern) {
        DecimalFormat format = new DecimalFormat(pattern, new DecimalFormatSymbols(locale));
        return format.format(number);
    }

    /**
     * 数値(double)を引数で指定される書式と指定の数値変換の記号セットで文字列に変換します。
     * 
     * @param symbols
     *            数値変換の記号セット
     * @param number
     *            数値
     * @param pattern
     *            書式
     * @return 変換後の文字列
     */
    public static String format(DecimalFormatSymbols symbols, double number, String pattern) {
        DecimalFormat format = new DecimalFormat(pattern, symbols);
        return format.format(number);
    }

    /*
     * validation
     */

    /**
     * 数値文字列として認識できるか評価します。
     * 
     * @param num
     *            評価する数値文字列
     * @param pattern
     *            文字列の書式パターン
     * @return 数値文字列として認識できる場合のみ<code>true</code>
     */
    public static boolean isDecimal(CharSequence num, String pattern) {
        return isDecimal(Locale.getDefault(), num, pattern);
    }

    /**
     * 数値文字列として認識できるか評価します。
     * 
     * @param locale
     *            ロケール
     * @param num
     *            評価する数値文字列
     * @param pattern
     *            文字列の書式パターン
     * @return 数値文字列として認識できる場合のみ<code>true</code>
     */
    public static boolean isDecimal(Locale locale, CharSequence num, String pattern) {
        return isDecimal(new DecimalFormatSymbols(locale), num, pattern);
    }

    /**
     * 数値文字列として認識できるか評価します。
     * 
     * @param symbols
     *            数値変換の記号セット
     * @param num
     *            評価する数値文字列
     * @param pattern
     *            文字列の書式パターン
     * @return 数値文字列として認識できる場合のみ<code>true</code>
     */
    public static boolean isDecimal(DecimalFormatSymbols symbols, CharSequence num, String pattern) {
        if (num == null) {
            return false;
        }
        try {
            DecimalFormat format = new DecimalFormat(pattern, symbols);
            format.parse(num.toString());
        } catch (ParseException e) {
            return false;
        }
        return true;
    }

    /**
     * 一つ以上の書式パターンで数値文字列として認識できるか評価します。
     * 
     * @param num
     *            評価する数値文字列
     * @param patterns
     *            文字列の書式パターンの配列
     * @return 一つ以上の書式パターンで数値文字列として認識できる場合のみ<code>true</code>
     */
    public static boolean isDecimal(CharSequence num, String[] patterns) {
        return isDecimal(Locale.getDefault(), num, patterns);
    }

    /**
     * 一つ以上の書式パターンで数値文字列として認識できるか評価します。
     * 
     * @param locale
     *            ロケール
     * @param num
     *            評価する数値文字列
     * @param patterns
     *            文字列の書式パターンの配列
     * @return 一つ以上の書式パターンで数値文字列として認識できる場合のみ<code>true</code>
     */
    public static boolean isDecimal(Locale locale, CharSequence num, String[] patterns) {
        return isDecimal(new DecimalFormatSymbols(locale), num, patterns);
    }

    /**
     * 一つ以上の書式パターンで数値文字列として認識できるか評価します。
     * 
     * @param symbols
     *            数値変換の記号セット
     * @param num
     *            評価する数値文字列
     * @param patterns
     *            文字列の書式パターンの配列
     * @return 一つ以上の書式パターンで数値文字列として認識できる場合のみ<code>true</code>
     */
    public static boolean isDecimal(DecimalFormatSymbols symbols, CharSequence num, String[] patterns) {
        if (num == null) {
            return false;
        }
        String snum = num.toString();
        DecimalFormat format = new DecimalFormat();
        format.setDecimalFormatSymbols(symbols);
        for (int i = 0; i < patterns.length; i++) {
            try {
                format.applyPattern(patterns[i]);
                format.parse(snum);
                return true;
            } catch (ParseException e) {
                // false;
            }
        }
        return false;
    }
}
