/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.util;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import shohaku.core.helpers.Eval;

/**
 * 拡張可能な階層化リソースバンドルの抽象実装を提供します。 <br>
 * この拡張可能なリソースバンドルは<code>java.util.ResourceBundle</code>同様に地域対応に主眼を置きますが、その用途に制限を持ちません。 <br>
 * 論理的には階層化されたリソースは全て実装の対象とすることが出来ます。 <br>
 * <br>
 * リソースを定義するファイル形式には<code>XResourceBundleCreater</code>で実装出来る任意の形式を利用できます。<br>
 * 階層化されたリソースをキャッシュし束ねるには <code>XResourceBundleCache</code>を使用します。 <br>
 * またリソースを束ね階層化する基準とファイル名の規約には <code>XResourceBundleBaseCalculator</code> の実装として任意に定義出来ます。 <br>
 * そのため<code>java.util.ResourceBundle</code>に於ける<code>java.util.Locale</code>のファイル名規約のと同等のファイル接尾辞を設計し、<br>
 * それを示すクラスを定義する事で、容易に独自基準で階層化されたリソースバンドルを構築出来ます。 <br>
 * <br>
 * 参照実装として、{@link shohaku.core.util.CELResourceBundle} を参照してください。
 * 
 * @see shohaku.core.util.CELResourceBundle
 */
public abstract class XResourceBundle {

    /** リソースを格納します。 */
    protected final Map lookupMap;

    /** リソースキーを格納します。 */
    protected final Set lookupKeys;

    /** 親バンドルを格納します。 */
    protected final XResourceBundle parent;

    /** 束縛基準を格納します。 */
    protected final Object bundleBase;

    /**
     * リソースリストを初期化します。
     * 
     * @param parent
     *            親バンドル
     * @param bundleBase
     *            束縛基準
     * @param resources
     *            リソース
     */
    protected XResourceBundle(XResourceBundle parent, Object bundleBase, Map resources) {
        this.parent = parent;
        this.bundleBase = bundleBase;
        this.lookupMap = resources;
        this.lookupKeys = lookupMap.keySet();
    }

    /*
     * utils
     */

    /**
     * 束縛基準を返却します。
     * 
     * @return 束縛基準
     */
    protected Object getBundleBase() {
        return bundleBase;
    }

    /*
     * Get Resources
     */

    /**
     * このバンドルと上位階層の全てのリソースキーの反復子を返却します。 <br>
     * この反復子は削除不可です。
     * 
     * @return このバンドルと上位階層の全てのリソースキーの反復子
     */
    public Iterator keyIterator() {
        Iterator parentIterator = (this.parent != null) ? this.parent.keyIterator() : null;
        return new XResourceBundleIterator(lookupKeys, parentIterator);
    }

    /**
     * リソースキーが示す値を返却します。指定されたキーが存在しない場合<code>null</code>を返却します。
     * 
     * @param key
     *            リソースキー
     * @throws NullPointerException
     *             <code>key</code>が<code>null</code>の場合
     * @return リソースキーが示す値
     */
    public Object getObject(String key) {
        if (key == null) {
            throw new NullPointerException("resource key as null");
        }
        Object obj = lookupMap.get(key);
        if (obj == null) {
            if (parent != null) {
                obj = parent.getObject(key);
            }
        }
        return obj;
    }

    /**
     * 指定された文字列型のキーが示す値を返却します。 <br>
     * 指定されたキーが存在しない場合<code>defaultValue</code>を返却します。
     * 
     * @param key
     *            リソースキー
     * @param defaultValue
     *            リソースキー
     * @return リソースキーが示す値
     * @throws NullPointerException
     *             <code>key</code>が<code>null</code>の場合
     */
    public Object getObject(String key, Object defaultValue) {
        if (containsKey(key)) {
            return getObject(key);
        } else {
            return defaultValue;
        }
    }

    /**
     * 指定されたキーがリソースセットに含まれている場合に<code>true</code>を返却します。 <br>
     * 親の拡張リソースバンドルも検索対象とします。
     * 
     * @param key
     *            リソースキー
     * @return 指定されたキーが含まれている場合 true
     * @throws NullPointerException
     *             <code>key</code>が<code>null</code>の場合
     */
    public boolean containsKey(String key) {
        if (key == null) {
            throw new NullPointerException("resource key as null");
        }
        if (!lookupMap.containsKey(key)) {
            if (parent != null) {
                return lookupMap.containsKey(key);
            }
            return false;
        }
        return true;
    }

    /**
     * 指定されたキーがリソースセットに含まれていると同時に、値が指定されたクラスとキャスト可能な関係に有る場合に<code>true</code>を返却します。 <br>
     * 親の拡張リソースバンドルも検索対象とします。値が<code>null</code>の場合キャスト可能<code>true</code>を返却します。
     * 
     * @param key
     *            リソースキー
     * @param type
     *            キャスト可能な関係に有るか検証するクラス
     * @return 指定されたキーが含まれている場合 true
     * @throws NullPointerException
     *             <code>key</code>か<code>type</code>が<code>null</code>の場合
     */
    public boolean containsKey(String key, Class type) {
        if (Eval.isOrNull(key, type)) {
            throw new NullPointerException("resource key or type as null");
        }
        if (containsKey(key)) {
            Object o = getObject(key);
            if (o != null) {
                return type.isInstance(o);
            }
            return true;
        }
        return false;
    }

}