/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.util;

import java.io.InputStream;
import java.util.Locale;

import shohaku.core.lang.NoSuchResourceException;
import shohaku.core.lang.ResourceLoader;

/**
 * 拡張可能な階層化リソースバンドルのヘルパーメソッドを提供します。
 */
public class XResourceBundleHelper {

    /**
     * バンドル名から束縛基準としてのロケールを算出して返却します。
     * 
     * @param baseName
     *            拡張リソースバンドルの基底名
     * @param bundleName
     *            基底名にロケール情報を付加したバンドル名
     * @return バンドル名から算出されるロケール
     */
    public static Object calculateLocaleBundleBase(String baseName, String bundleName) {
        if (baseName.length() == bundleName.length()) {
            return new Locale("", "");
        } else if (baseName.length() < bundleName.length()) {
            int pos = baseName.length();
            String temp = bundleName.substring(pos + 1);
            pos = temp.indexOf('_');
            if (pos == -1) {
                return new Locale(temp, "", "");
            }

            String language = temp.substring(0, pos);
            temp = temp.substring(pos + 1);
            pos = temp.indexOf('_');
            if (pos == -1) {
                return new Locale(language, temp, "");
            }

            String country = temp.substring(0, pos);
            temp = temp.substring(pos + 1);

            return new Locale(language, country, temp);
        } else {
            throw new IllegalArgumentException("baseName:" + baseName + ", bundleName:" + bundleName);
        }
    }

    /**
     * ロケールを束縛基準とするバンドルのデフォルトの束縛基準を返却します。
     * 
     * @return ロケールのデフォルトの束縛基準
     */
    public static Object getLocaleDefaultBundleBase() {
        return Locale.getDefault();
    }

    /* ロケールを束縛基準とする検索候補となるバンドル名の最大数。 */
    private static final int MAX_BUNDLES_SEARCHED = 4;

    /* ロケールを束縛基準とする基底名のインデックス。 */
    private static final int BASE_INDEX = 0;

    /* ロケールを束縛基準とする言語を持つバンドル名のインデックス。 */
    private static final int LANGUAGE_INDEX = 1;

    /* ロケールを束縛基準とする国を持つバンドル名のインデックス。 */
    private static final int COUNTRY_INDEX = 2;

    /* ロケールを束縛基準とするバリアントを持つバンドル名のインデックス。 */
    private static final int VARIANT_INDEX = 3;

    /**
     * ロケールを束縛基準とするバンドルの検索候補となるバンドル名を算出して返却します。
     * 
     * @param baseName
     *            基底名
     * @param bundleBase
     *            束縛基準（ロケール）
     * @return 候補となるバンドル名
     */
    public static String[] candidateLocaleBundleNames(String baseName, Object bundleBase) {

        final Locale locale = (Locale) bundleBase;
        final String[] result = new String[MAX_BUNDLES_SEARCHED];
        final String language = locale.getLanguage();
        final int languageLength = language.length();
        final String country = locale.getCountry();
        final int countryLength = country.length();
        final String variant = locale.getVariant();
        final int variantLength = variant.length();

        result[BASE_INDEX] = baseName;

        if (languageLength + countryLength + variantLength == 0) {
            // The locale is "", "", "".
            return result;
        }
        final StringBuffer temp = new StringBuffer(baseName);
        temp.append('_');
        temp.append(language);
        if (languageLength > 0) {
            result[LANGUAGE_INDEX] = temp.toString();
        }

        if (countryLength + variantLength == 0) {
            return result;
        }
        temp.append('_');
        temp.append(country);
        if (countryLength > 0) {
            result[COUNTRY_INDEX] = temp.toString();
        }

        if (variantLength == 0) {
            return result;
        }
        temp.append('_');
        temp.append(variant);
        result[VARIANT_INDEX] = temp.toString();

        return result;
    }

    /**
     * ロケールを束縛基準とするバンドルの検索候補となるバンドル名を算出して返却します。
     * 
     * @param bundleBase
     *            束縛基準（ロケール）
     * @return 候補となるバンドル名
     */
    public static String candidateLocaleBundleId(Object bundleBase) {

        final Locale locale = (Locale) bundleBase;
        final String language = locale.getLanguage();
        final int languageLength = language.length();
        final String country = locale.getCountry();
        final int countryLength = country.length();
        final String variant = locale.getVariant();
        final int variantLength = variant.length();

        if (languageLength + countryLength + variantLength == 0) {
            // The locale is "", "", "".
            return "root";
        }
        final StringBuffer sb = new StringBuffer();
        sb.append(language);
        if (countryLength + variantLength > 0) {
            sb.append('_');
            sb.append(country);
            if (variantLength > 0) {
                sb.append('_');
                sb.append(variant);
            }
        }
        return sb.toString().toLowerCase();
    }

    /**
     * リソースファイルの入力ストリームを生成して返却します。 <br>
     * リソースファイルを発見出来なかった場合<code>null</code>を返却します。
     * 
     * @param loader
     *            クラスリーダ
     * @param bundleName
     *            バンドル名
     * @return リソースファイルの入力ストリーム
     */
    public static InputStream getPropertiesBundleInputStream(ClassLoader loader, String bundleName) {

        // XMLファイルパスに変換します。
        final String resName = bundleName.replace('.', '/') + ".properties";

        // 特権を用いて強欲に検索します。
        InputStream stream = null;
        try {
            stream = ResourceLoader.getResourceAsStream(resName, loader);
        } catch (NoSuchResourceException e) {
            // no op
        }
        if (stream != null) {
            // make sure it is buffered
            return new java.io.BufferedInputStream(stream);
        }
        // 発見できなかった場合<code>null</code>を返却します。
        return null;
    }

}
