/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.util.cel;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import shohaku.core.helpers.Seek;
import shohaku.core.lang.Boxing;

/**
 * リテラル文字を区切りとして定義される式言語を任意に束ね、実行する簡易式言語の実行機能を提供します。 <br>
 * <br>
 * <code>CEL</code>は<code>(Composite CELExpression Language)</code>の略です。
 * <p>
 * この言語はインタープリタであり、完全に実行時にコンパイルされます。 <br>
 * この言語は速度や機能の豊富さ、厳密性等よりも、柔軟性と簡易性を重視して設計されています。 <br>
 * 厳密性や高度な機能を実現する場合や、繰り返し実行する場合等の用途には向いていません。 <br>
 * 平易な式を一回または数回処理する用途で使用する事が最適です。
 * </p>
 * <p>
 * リテラル文字の衝突しない「式」を入れ子状に組み込む事が出来ます。また束ねる「式」に応じて機能を増減出来ます。機能拡張や機能縮小も容易です。
 * </p>
 */
public class CELBinder {

    /* このインスタンスで使用する式のマッピング情報。 */
    private final Map expressionMap;

    /* このインスタンスで使用する式。 */
    private final CELExpression[] expressions;

    /* このインスタンスで使用するロケール。 */
    private final Locale locale;

    /**
     * デフォルトのロケールと空の式で初期化します。
     */
    public CELBinder() {
        this(Locale.getDefault(), new CELExpression[0]);
    }

    /**
     * 式とデフォルトのロケールで初期化します。
     * 
     * @param exps
     *            式
     */
    public CELBinder(CELExpression[] exps) {
        this(Locale.getDefault(), exps, toMappingExpressions(exps));
    }

    /**
     * ロケールと式を指定して初期化します。
     * 
     * @param locale
     *            ロケール
     * @param exps
     *            式
     */
    public CELBinder(Locale locale, CELExpression[] exps) {
        this(locale, exps, toMappingExpressions(exps));
    }

    /* フォーマッタを初期化します。 */
    private CELBinder(Locale locale, CELExpression[] exps, Map fsm) {
        this.expressions = exps;
        this.locale = locale;
        this.expressionMap = fsm;
    }

    /**
     * 登録されている式一覧を返却します．
     * 
     * @return 登録されている式一覧
     */
    public CELExpression[] getExpressions() {
        return (CELExpression[]) expressions.clone();
    }

    /**
     * ロケールを返却します．
     * 
     * @return ロケール
     */
    public Locale getLocale() {
        return locale;
    }

    /**
     * 式変換を実行します。
     * 
     * @param expression
     *            式構文
     * @return 式の結果
     */
    public Object getValue(String expression) {
        return getValue(expression, null);
    }

    /**
     * 式変換を実行します。
     * 
     * @param expression
     *            式構文
     * @param context
     *            式構文から参照する値
     * @return 式の結果
     */
    public Object getValue(String expression, CELContext context) {
        return getValue(expression, 0, context);
    }

    /**
     * 式変換を実行します。
     * 
     * @param expression
     *            式構文
     * @param beginIndex
     *            式構文中の式の開始インデックス
     * @param context
     *            式構文から参照するコンテキスト
     * @return 式の結果
     */
    public Object getValue(String expression, int beginIndex, CELContext context) {
        CELReturnValue rValue = getReturnValue(expression, beginIndex, context);
        return rValue.value;
    }

    /**
     * 式変換を実行します。
     * 
     * @param expression
     *            式構文
     * @param beginIndex
     *            式構文中の式の開始インデックス
     * @param context
     *            式構文から参照する値
     * @return 式の結果
     */
    public CELReturnValue getReturnValue(String expression, int beginIndex, CELContext context) {
        return execute(expression, beginIndex, context);
    }

    /* リテラル文字を持たないリテラルクラス。 */
    private static final Character NO_LITERAL = Boxing.box('\u0000');

    /* 個々の要素の式変換を実行します。 */
    private CELReturnValue execute(String expression, int beginIndex, CELContext context) {
        beginIndex = Seek.skipSp(expression, beginIndex);
        final char beginLiteral = expression.charAt(beginIndex);
        CELExpression exp = (CELExpression) expressionMap.get(Boxing.box(beginLiteral));
        if (exp == null) {
            exp = (CELExpression) expressionMap.get(NO_LITERAL);
            return exp.execute(this, expression, beginIndex, context);
        } else {
            return exp.execute(this, expression, beginIndex, context);
        }

    }

    /*
     * static
     */

    /* 式を識別子単位でマッピングして返却します。 */
    private static Map toMappingExpressions(CELExpression[] exps) {
        Map m = new HashMap((int) (exps.length * 1.5));
        for (int i = 0; i < exps.length; i++) {
            CELLiteral literal = exps[i].getLiteral();
            m.put(Boxing.box(literal.begin), exps[i]);
        }
        return Collections.unmodifiableMap(m);
    }

    /*
     * Default Binder
     */

    /* 基本型を生成する式。 */
    private static final CELExpression[] baseTypeCreationExpressions;
    static {
        List exps = new ArrayList();
        exps.add(new NoLiteralExpression());
        exps.add(new CharacterExpression());
        exps.add(new StringExpression());
        exps.add(new DateTimeExpression());
        exps.add(new RegexPatternExpression());
        baseTypeCreationExpressions = (CELExpression[]) exps.toArray(new CELExpression[0]);
    }

    /* 基本型を生成する式。 */
    private static final CELBinder baseTypeCreationBinder = new CELBinder(baseTypeCreationExpressions);

    /**
     * 基本型の生成式バインダーを返却します。 <br>
     * プリミティブ型、文字列型、日付型、正規表現の生成機能をを提供しています。
     * 
     * @return 基本型の生成式バインダー
     */
    public static CELBinder getBaseTypeCreationBinder() {
        return baseTypeCreationBinder;
    }

    /* コレクション型を生成する式。 */
    private static final CELExpression[] collectionCreationExpressions;
    static {
        List exps = new ArrayList();
        exps.add(new NoLiteralExpression());
        exps.add(new CharacterExpression());
        exps.add(new StringExpression());
        exps.add(new DateTimeExpression());
        exps.add(new RegexPatternExpression());
        exps.add(new ListExpression());
        exps.add(new SetExpression());
        exps.add(new MapExpression());
        collectionCreationExpressions = (CELExpression[]) exps.toArray(new CELExpression[0]);
    }

    /* コレクション型を生成する式。 */
    private static final CELBinder collectionCreationBinder = new CELBinder(collectionCreationExpressions);

    /**
     * コレクション型が生成可能なバインダーを返却します。 <br>
     * 基本型に加え、コレクション（Map, Set, List）の生成機能をを提供しています。
     * 
     * @return コレクション型が生成可能なバインダー
     */
    public static CELBinder getCollectionCreationBinder() {
        return collectionCreationBinder;
    }

    /* 大半のオブジェクト型が生成可能な式。 */
    private static final CELExpression[] objectCreationExpressions;
    static {
        List exps = new ArrayList();
        exps.add(new NoLiteralExpression());
        exps.add(new CharacterExpression());
        exps.add(new StringExpression());
        exps.add(new DateTimeExpression());
        exps.add(new RegexPatternExpression());
        exps.add(new ListExpression());
        exps.add(new SetExpression());
        exps.add(new MapExpression());
        exps.add(new ObjectCreateExpression());
        exps.add(new ReferenceExpression());
        exps.add(new IntrospectExpression());
        objectCreationExpressions = (CELExpression[]) exps.toArray(new CELExpression[0]);
    }

    /* 大半のオブジェクト型が生成可能な式。 */
    private static final CELBinder objectCreationBinder = new CELBinder(objectCreationExpressions);

    /**
     * 大半のオブジェクト型が生成可能なバインダーを返却します。 <br>
     * コレクション型に加え、インスタンス生成式を提供しています。
     * 
     * @return 大半のオブジェクト型が生成可能なバインダー
     */
    public static CELBinder getObjectCreationBinder() {
        return objectCreationBinder;
    }

}
