/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to
 * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.core.util.cel;

import shohaku.core.helpers.Cutter;
import shohaku.core.helpers.Eval;
import shohaku.core.helpers.Seek;

/**
 * リテラル文字を持たない特殊式を提供します、null値, 論理値, プリミティブの数値を生成します。
 */
public class NoLiteralExpression implements CELExpression {

    /* この式のリテラル文字を定義します。 */
    private static final CELLiteral literal = new CELLiteral('\u0000', '\u0000');

    /* 数値に使用される文字を定義します。 */
    private static final char[] NUM_CHARS = "-+xX0123456789abcdefABCDEF.BSILFD".toCharArray();

    /* 数値の開始位置に使用される文字を定義します。 */
    private static final char[] NUM_BEGIN_CHARS = "-+0123456789.".toCharArray();

    /* 数値に使用される文字を定義します。 */
    private static final char[] REF_CHARS = "_-:abcdefghijklnmopqrstuvwxyzABCDEFGHIJKLNMOPQRSTUVWXYZ0987654321"
            .toCharArray();

    /* 数値の開始位置に使用される文字を定義します。 */
    private static final char[] REF_BEGIN_CHARS = "_abcdefghijklnmopqrstuvwxyzABCDEFGHIJKLNMOPQRSTUVWXYZ".toCharArray();

    /**
     * リテラル文字を持たない特殊リテラル '\u0000', '\u0000' を返却します。
     * 
     * @return リテラル文字
     */
    public CELLiteral getLiteral() {
        return literal;
    }

    /**
     * 式を実行して値を返却します。
     * 
     * @param binder
     *            処理基のバインダー
     * @param expression
     *            式
     * @param beginIndex
     *            開始インデックス
     * @param context
     *            引数値の一覧
     * @return 式の結果
     */
    public CELReturnValue execute(CELBinder binder, String expression, int beginIndex, CELContext context) {

        int off = Seek.skipSp(expression, beginIndex);
        // Null
        if (Eval.isStartsWith(expression, "null", off)) {
            return new CELReturnValue(null, off += "null".length());
        }
        // Boolean.TRUE
        if (Eval.isStartsWith(expression, "true", off)) {
            return new CELReturnValue(Boolean.TRUE, off += "true".length());
        }
        // Boolean.FALSE
        if (Eval.isStartsWith(expression, "false", off)) {
            return new CELReturnValue(Boolean.FALSE, off += "false".length());
        }

        if (Eval.isContains(NUM_BEGIN_CHARS, expression.charAt(off))) {
            return executeNumber(expression, off);
        } else if (Eval.isContains(REF_BEGIN_CHARS, expression.charAt(off))) {
            return executeReference(expression, off, context);
        } else {
            throw new CELFormatException("format err :" + expression);
        }

    }

    private CELReturnValue executeReference(String expression, int off, CELContext context) {
        // Reference
        int endIndex = Seek.skip(expression, REF_CHARS, off);
        Object ref = null;
        String refId = expression.substring(off, endIndex);
        if (context.containsName(refId)) {
            ref = context.getValue(refId);
        } else {
            throw new CELFormatException("format err, no contains id:" + expression);
        }
        return new CELReturnValue(ref, endIndex);
    }

    private CELReturnValue executeNumber(String expression, int off) {
        // Number
        int endIndex = Seek.skip(expression, NUM_CHARS, off);
        Object num = null;
        String exp = expression.substring(off, endIndex);
        try {
            char numLiteral = Cutter.end(exp);
            String snum = Cutter.cut(exp, 0, 1);
            switch (numLiteral) {
            case 'B':
                num = Byte.decode(snum);
                break;
            case 'S':
                num = Short.decode(snum);
                break;
            case 'I':
                num = Integer.decode(snum);
                break;
            case 'L':
                num = Long.decode(snum);
                break;
            case 'F':
                num = Float.valueOf(snum);
                break;
            case 'D':
                num = Double.valueOf(snum);
                break;
            default:
                if (Eval.isContains(exp, '.')) {
                    num = Double.valueOf(exp);
                } else {
                    num = Integer.decode(exp);
                }
            }
        } catch (NumberFormatException e) {
            throw new CELFormatException("format err :" + exp, e);
        }
        return new CELReturnValue(num, endIndex);
    }
}