/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.core.util.cel;

import java.lang.reflect.Array;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import shohaku.core.beans.BeanUtilities;
import shohaku.core.beans.InvocationBeansException;
import shohaku.core.helpers.Cutter;
import shohaku.core.helpers.Eval;
import shohaku.core.helpers.Seek;
import shohaku.core.lang.LangUtils;
import shohaku.core.lang.NoSuchResourceException;

/**
 * コンストラクタ、スタティックメソッドからのオブジェクト生成や配列の生成、クラス参照の取得を行う式を提供します。 <br>
 * また配列の初期化や生成されたオブジェクトへのプロパティ設定を行う機能も提供しています。
 */
public class ObjectCreateExpression implements CELExpression {

    /* この式のリテラル文字を定義します。 */
    private static final CELLiteral literal = new CELLiteral('@', '@');

    /**
     * リテラル文字 '@', '@' を返却します。
     * 
     * @return リテラル文字
     */
    public CELLiteral getLiteral() {
        return literal;
    }

    /**
     * 式を実行して値を返却します。
     * 
     * @param binder
     *            処理基のバインダー
     * @param expression
     *            式
     * @param beginIndex
     *            開始インデックス
     * @param context
     *            引数値の一覧
     * @return 式の結果
     */
    public CELReturnValue execute(CELBinder binder, String expression, int beginIndex, CELContext context) {
        final char[] nextLiterals = "(#{@".toCharArray();
        final char[] staticNextLiterals = "(@".toCharArray();

        String exp = expression;
        int off = ++beginIndex;
        Class type = null;
        String className = null;
        String staticMethodName = null;
        String staticFieldName = null;
        off = Seek.skipSp(exp, off);

        // #=Static Method or Static Field, (=Constructor, {=Array, @=Class
        int nextIndex = Seek.orIndexOf(exp, nextLiterals, off);
        if (Eval.isNegative(nextIndex)) {
            throw new CELFormatException("format err:" + exp);
        }

        char offLiteral = exp.charAt(nextIndex);

        // switch create type
        switch (offLiteral) {
        case '#':
            // Static Method or Static Field
            className = exp.substring(off, nextIndex);
            int staticNextIndex = Seek.orIndexOf(exp, staticNextLiterals, nextIndex);
            if (Eval.isPositive(staticNextIndex)) {
                if (Eval.isCharAt(exp, staticNextIndex, '(')) {
                    staticMethodName = exp.substring((nextIndex + 1), staticNextIndex);
                    offLiteral = '(';
                } else {
                    staticFieldName = Cutter.trimEnd(exp.substring((nextIndex + 1), staticNextIndex));
                    offLiteral = '@';
                }
                off = staticNextIndex;
            } else {
                throw new CELFormatException("format err:" + exp);
            }
            break;
        case '(':
            // Constructor
            className = exp.substring(off, nextIndex);
            off = nextIndex;
            break;

        default:
            // Array or Class
            className = Cutter.trimEnd(exp.substring(off, nextIndex));
            off = nextIndex;
            break;
        }

        // get class
        try {
            type = LangUtils.loadClass(className, null);
        } catch (NoSuchResourceException e) {
            throw new CELFormatException("bean invocation err:" + exp, e);
        }

        // get value
        Object o = null;
        // switch create type
        if (offLiteral == literal.end) {

            // Static Field or Class
            if (staticFieldName != null) {
                try {
                    o = BeanUtilities.getFieldValue(type, null, staticFieldName);
                } catch (InvocationBeansException e) {
                    throw new CELFormatException("bean invocation err:" + exp, e);
                }
            } else {
                o = type;
            }
            off = Seek.skipSp(exp, off);

        } else if (offLiteral == '{') {

            // Array
            if (!Eval.isArrayClass(type)) {
                throw new CELFormatException("no array type:" + exp);
            }
            CELReturnValue rValue = CELHelper.getArrayListReturnValue(binder, exp, off, context, '{', '}');
            List elems = (List) rValue.value;
            off = rValue.nextIndex;
            Object a;
            try {
                a = Array.newInstance(type.getComponentType(), elems.size());
                int index = 0;
                for (Iterator i = elems.iterator(); i.hasNext();) {
                    Array.set(a, index, i.next());
                    index++;
                }
            } catch (NegativeArraySizeException e) {
                throw new CELFormatException("array create err:" + exp, e);
            } catch (IllegalArgumentException e) {
                throw new CELFormatException("array create err:" + exp, e);
            }
            o = a;
            off = Seek.skipSp(exp, off);

        } else if (offLiteral == '(') {
            // Static Method or Constructor
            CELReturnValue rValue = CELHelper.getArguments(binder, exp, off, context);
            Object[] args = (Object[]) rValue.value;
            off = rValue.nextIndex;
            try {
                if (staticMethodName != null) {
                    o = BeanUtilities.invokeMethod(type, null, staticMethodName, args);
                } else {
                    o = BeanUtilities.newInstance(type, args);
                }
            } catch (InvocationBeansException e) {
                throw new CELFormatException("bean invocation err:" + exp, e);
            }
            if (o instanceof Void) {
                throw new CELFormatException("method return type void:" + exp);
            }
            off = Seek.skipSp(exp, off);

            // set properties
            if (!Eval.isCharAt(exp, off, literal.end)) {
                off = doSetProperties(binder, exp, off, o, context);
            }
        } else {
            throw new CELFormatException("format err:" + exp);
        }

        // is end literal
        if (!Eval.isCharAt(exp, off, literal.end)) {
            throw new CELFormatException("format err:" + exp);
        }

        return new CELReturnValue(o, ++off);// next index
    }

    private int doSetProperties(CELBinder binder, String exp, int off, Object o, CELContext context) {
        CELReturnValue rPropValue = CELHelper.getLinkedHashMapReturnValue(binder, exp, off, context, '{', '}');
        Map props = (Map) rPropValue.value;
        off = rPropValue.nextIndex;
        if (!props.isEmpty()) {
            try {
                BeanUtilities.setProperties(o, props);
            } catch (InvocationBeansException e) {
                throw new CELFormatException("format err:" + exp, e);
            }
        }
        off = Seek.skipSp(exp, off);
        return off;
    }

}
